from _typeshed import Incomplete

from kafka.protocol.api import Request, Response

class TxnOffsetCommitResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class TxnOffsetCommitResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class TxnOffsetCommitResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class TxnOffsetCommitRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = TxnOffsetCommitResponse_v0
    SCHEMA: Incomplete

class TxnOffsetCommitRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = TxnOffsetCommitResponse_v1
    SCHEMA: Incomplete

class TxnOffsetCommitRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = TxnOffsetCommitResponse_v2
    SCHEMA: Incomplete

TxnOffsetCommitRequest: Incomplete
TxnOffsetCommitResponse: Incomplete



from _typeshed import Incomplete
from typing import TypedDict, type_check_only

from braintree.graphql.inputs.monetary_amount_input import _GraphqlVariables as _MonetaryAmountGraphqlVariables
from braintree.graphql.inputs.payer_info_input import _GraphqlVariables as _PayerInfoGraphqlVariables

@type_check_only
class _PaymentContext(TypedDict, total=False):
    amount: _MonetaryAmountGraphqlVariables
    cancelUrl: str
    countryCode: str
    expiryDate: str
    merchantAccountId: str
    orderId: str
    payerInfo: _PayerInfoGraphqlVariables
    returnUrl: str
    type: str

@type_check_only
class _GraphqlVariables(TypedDict):
    paymentContext: _PaymentContext

class CreateLocalPaymentContextInput:
    def __init__(
        self,
        amount: dict[str, Incomplete] | None = None,
        cancel_url: str | None = None,
        country_code: str | None = None,
        expiry_date: str | None = None,
        merchant_account_id: str | None = None,
        order_id: str | None = None,
        payer_info: dict[str, Incomplete] | None = None,
        return_url: str | None = None,
        type: str | None = None,
    ) -> None: ...
    def to_graphql_variables(self) -> _GraphqlVariables: ...



from _typeshed import Incomplete, SupportsWrite
from collections.abc import Iterable
from typing import TypeVar

from pygments.formatter import Formatter
from pygments.token import _TokenType

_T = TypeVar("_T", str, bytes)

__all__ = ["RtfFormatter"]

class RtfFormatter(Formatter[_T]):
    fontface: Incomplete
    fontsize: Incomplete
    @staticmethod
    def hex_to_rtf_color(hex_color: str) -> str: ...
    def format_unencoded(self, tokensource: Iterable[tuple[_TokenType, str]], outfile: SupportsWrite[str]) -> None: ...



from typing import Final

class TclTkInfo:
    TCL_ROOTNAME: Final = "_tcl_data"
    TK_ROOTNAME: Final = "_tk_data"
    def __init__(self) -> None: ...
    available: bool
    tkinter_extension_file: str | None
    tcl_version: tuple[int, int] | None
    tk_version: tuple[int, int] | None
    tcl_threaded: bool
    tcl_data_dir: str | None
    tk_data_dir: str | None
    tcl_module_dir: str | None
    is_macos_system_framework: bool
    tcl_shared_library: str | None
    tk_shared_library: str | None
    data_files: list[tuple[str, str, str]]

tcltk_info: TclTkInfo



import threading
from _typeshed import Incomplete
from typing import NamedTuple

log: Incomplete

class PartitionResponse(NamedTuple):
    error: Incomplete = ...
    base_offset: Incomplete = ...
    last_offset: Incomplete = ...
    log_append_time: Incomplete = ...
    log_start_offset: Incomplete = ...
    record_errors: Incomplete = ...
    error_message: Incomplete = ...
    current_leader: Incomplete = ...

class Sender(threading.Thread):
    DEFAULT_CONFIG: Incomplete
    config: Incomplete
    name: Incomplete
    def __init__(self, client, metadata, accumulator, **configs) -> None: ...
    def run(self) -> None: ...
    def run_once(self) -> None: ...
    def initiate_close(self) -> None: ...
    def force_close(self) -> None: ...
    def add_topic(self, topic) -> None: ...
    def wakeup(self) -> None: ...
    def bootstrap_connected(self): ...

class SenderMetrics:
    metrics: Incomplete
    batch_size_sensor: Incomplete
    compression_rate_sensor: Incomplete
    queue_time_sensor: Incomplete
    records_per_request_sensor: Incomplete
    byte_rate_sensor: Incomplete
    retry_sensor: Incomplete
    error_sensor: Incomplete
    max_record_size_sensor: Incomplete
    def __init__(self, metrics, client, metadata) -> None: ...
    def add_metric(
        self, metric_name, measurable, group_name: str = "producer-metrics", description=None, tags=None, sensor_name=None
    ) -> None: ...
    def maybe_register_topic_metrics(self, topic): ...
    def update_produce_request_metrics(self, batches_map) -> None: ...
    def record_retries(self, topic, count) -> None: ...
    def record_errors(self, topic, count) -> None: ...



import abc
from _typeshed import Incomplete

class AbstractType(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def encode(cls, value): ...
    @abc.abstractmethod
    def decode(cls, data): ...
    repr: Incomplete



from typing import Final, Literal, overload

import _win32typing

def OpenPrinter(printer: str, Defaults: _win32typing.PrinterDefaults | None = None, /) -> _win32typing.PyPrinterHANDLE: ...

@overload
def GetPrinter(hPrinter: _win32typing.PyPrinterHANDLE, /) -> _win32typing.PrinterInfo2Tuple: ...
@overload
def GetPrinter(hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[1], /) -> _win32typing.PrinterInfo1: ...
@overload
def GetPrinter(hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[2], /) -> _win32typing.PrinterInfo2: ...
@overload
def GetPrinter(hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[3], /) -> _win32typing.PrinterInfo3: ...
@overload
def GetPrinter(hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[4], /) -> _win32typing.PrinterInfo4: ...
@overload
def GetPrinter(hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[5], /) -> _win32typing.PrinterInfo5: ...
@overload
def GetPrinter(hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[6], /) -> _win32typing.PrinterInfo6: ...
@overload
def GetPrinter(hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[7], /) -> _win32typing.PrinterInfo7: ...
@overload
def GetPrinter(hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[8, 9], /) -> _win32typing.PrinterInfo89: ...

@overload
def SetPrinter(hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[0], pPrinter: int | None, Command: int, /) -> None: ...
@overload
def SetPrinter(
    hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[2], pPrinter: _win32typing.PrinterInfo2, Command: Literal[0], /
) -> None: ...
@overload
def SetPrinter(
    hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[3], pPrinter: _win32typing.PrinterInfo3, Command: Literal[0], /
) -> None: ...
@overload
def SetPrinter(
    hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[4], pPrinter: _win32typing.PrinterInfo4, Command: Literal[0], /
) -> None: ...
@overload
def SetPrinter(
    hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[5], pPrinter: _win32typing.PrinterInfo5, Command: Literal[0], /
) -> None: ...
@overload
def SetPrinter(
    hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[6], pPrinter: _win32typing.PrinterInfo6, Command: Literal[0], /
) -> None: ...
@overload
def SetPrinter(
    hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[7], pPrinter: _win32typing.PrinterInfo7, Command: Literal[0], /
) -> None: ...
@overload
def SetPrinter(
    hPrinter: _win32typing.PyPrinterHANDLE, Level: Literal[8, 9], pPrinter: _win32typing.PrinterInfo89, Command: Literal[0], /
) -> None: ...

def ClosePrinter(hPrinter: _win32typing.PyPrinterHANDLE, /) -> None: ...
def AddPrinterConnection(printer: str, /) -> None: ...
def DeletePrinterConnection(printer: str, /) -> None: ...

@overload
def EnumPrinters(flags: int, name: str | None = None, level: Literal[1] = 1, /) -> tuple[_win32typing.PrinterInfo1Tuple, ...]: ...
@overload
def EnumPrinters(flags: int, name: str | None, level: Literal[2], /) -> tuple[_win32typing.PrinterInfo2, ...]: ...
@overload
def EnumPrinters(flags: int, name: str | None, level: Literal[4], /) -> tuple[_win32typing.PrinterInfo4, ...]: ...
@overload
def EnumPrinters(flags: int, name: str | None, level: Literal[5], /) -> tuple[_win32typing.PrinterInfo5, ...]: ...

def GetDefaultPrinter() -> str: ...
def GetDefaultPrinterW() -> str: ...
def SetDefaultPrinter(printer: str, /) -> None: ...
def SetDefaultPrinterW(Printer: str | None, /) -> None: ...
def StartDocPrinter(
    hprinter: _win32typing.PyPrinterHANDLE, level: Literal[1], tuple: tuple[str, str | None, str | None], /
) -> int: ...
def EndDocPrinter(hPrinter: _win32typing.PyPrinterHANDLE, /) -> None: ...
def AbortPrinter(hPrinter: _win32typing.PyPrinterHANDLE, /) -> None: ...
def StartPagePrinter(hprinter: _win32typing.PyPrinterHANDLE, /) -> None: ...
def EndPagePrinter(hprinter: _win32typing.PyPrinterHANDLE, /) -> None: ...
def StartDoc(hdc: int, docinfo: tuple[str, str | None, str | None, int], /) -> int: ...
def EndDoc(hdc: int, /) -> None: ...
def AbortDoc(hdc: int, /) -> None: ...
def StartPage(hdc: int, /) -> None: ...
def EndPage(hdc: int, /) -> None: ...
def WritePrinter(hprinter: _win32typing.PyPrinterHANDLE, buf: bytes | bytearray | memoryview, /) -> int: ...

@overload
def EnumJobs(
    hPrinter: _win32typing.PyPrinterHANDLE, FirstJob: int, NoJobs: int, Level: Literal[1] = 1, /
) -> tuple[_win32typing.JobInfo1, ...]: ...
@overload
def EnumJobs(
    hPrinter: _win32typing.PyPrinterHANDLE, FirstJob: int, NoJobs: int, Level: Literal[2], /
) -> tuple[_win32typing.JobInfo2, ...]: ...
@overload
def EnumJobs(
    hPrinter: _win32typing.PyPrinterHANDLE, FirstJob: int, NoJobs: int, Level: Literal[3], /
) -> tuple[_win32typing.JobInfo3, ...]: ...

@overload
def GetJob(hPrinter: _win32typing.PyPrinterHANDLE, JobID: int, Level: Literal[1] = 1, /) -> _win32typing.JobInfo1: ...
@overload
def GetJob(hPrinter: _win32typing.PyPrinterHANDLE, JobID: int, Level: Literal[2], /) -> _win32typing.JobInfo2: ...
@overload
def GetJob(hPrinter: _win32typing.PyPrinterHANDLE, JobID: int, Level: Literal[3], /) -> _win32typing.JobInfo3: ...

@overload
def SetJob(hPrinter: _win32typing.PyPrinterHANDLE, JobID: int, Level: Literal[0], JobInfo: None, Command: int, /) -> None: ...
@overload
def SetJob(
    hPrinter: _win32typing.PyPrinterHANDLE, JobID: int, Level: Literal[1], JobInfo: _win32typing.JobInfo1, Command: int, /
) -> None: ...
@overload
def SetJob(
    hPrinter: _win32typing.PyPrinterHANDLE, JobID: int, Level: Literal[2], JobInfo: _win32typing.JobInfo2, Command: int, /
) -> None: ...
@overload
def SetJob(
    hPrinter: _win32typing.PyPrinterHANDLE, JobID: int, Level: Literal[3], JobInfo: _win32typing.JobInfo3, Command: int, /
) -> None: ...

def DocumentProperties(
    HWnd: int,
    hPrinter: _win32typing.PyPrinterHANDLE,
    DeviceName: str,
    DevModeOutput: _win32typing.PyDEVMODEW,
    DevModeInput: _win32typing.PyDEVMODEW,
    Mode: int,
    /,
) -> int: ...
def EnumPrintProcessors(Server: str | None = None, Environment: str | None = None, /) -> tuple[str, ...]: ...
def EnumPrintProcessorDatatypes(ServerName: str | None, PrintProcessorName: str, /) -> tuple[str, ...]: ...

@overload
def EnumPrinterDrivers(
    Server: str | None = None, Environment: str | None = None, Level: Literal[1] = 1, /
) -> tuple[_win32typing.DriverInfo1, ...]: ...
@overload
def EnumPrinterDrivers(
    Server: str | None, Environment: str | None, Level: Literal[2], /
) -> tuple[_win32typing.DriverInfo2, ...]: ...
@overload
def EnumPrinterDrivers(
    Server: str | None, Environment: str | None, Level: Literal[3], /
) -> tuple[_win32typing.DriverInfo3, ...]: ...
@overload
def EnumPrinterDrivers(
    Server: str | None, Environment: str | None, Level: Literal[4], /
) -> tuple[_win32typing.DriverInfo4, ...]: ...
@overload
def EnumPrinterDrivers(
    Server: str | None, Environment: str | None, Level: Literal[5], /
) -> tuple[_win32typing.DriverInfo5, ...]: ...
@overload
def EnumPrinterDrivers(
    Server: str | None, Environment: str | None, Level: Literal[6], /
) -> tuple[_win32typing.DriverInfo6, ...]: ...

def EnumForms(hprinter: _win32typing.PyPrinterHANDLE, /) -> tuple[_win32typing.FormInfo1, ...]: ...
def AddForm(hprinter: _win32typing.PyPrinterHANDLE, Form: _win32typing.FormInfo1, /) -> None: ...
def DeleteForm(hprinter: _win32typing.PyPrinterHANDLE, FormName: str, /) -> None: ...
def GetForm(hprinter: _win32typing.PyPrinterHANDLE, FormName: str, /) -> _win32typing.FormInfo1: ...
def SetForm(hprinter: _win32typing.PyPrinterHANDLE, FormName: str, Form: _win32typing.FormInfo1, /) -> None: ...
def AddJob(hprinter: _win32typing.PyPrinterHANDLE, /) -> tuple[str, int]: ...
def ScheduleJob(hprinter: _win32typing.PyPrinterHANDLE, JobId: int, /) -> None: ...

@overload
# DC_MINEXTENT, DC_MAXEXTENT
def DeviceCapabilities(  # type: ignore[overload-overlap]
    Device: str, Port: str, Capability: Literal[4, 5], DEVMODE: _win32typing.PyDEVMODEW | None = None, /
) -> _win32typing.PrinterExtents: ...
@overload
# DC_ENUMRESOLUTIONS
def DeviceCapabilities(  # type: ignore[overload-overlap]
    Device: str, Port: str, Capability: Literal[13], DEVMODE: _win32typing.PyDEVMODEW | None = None, /
) -> tuple[_win32typing.PrinterDpi, ...]: ...
@overload
# DC_PAPERS, DC_BINS, DC_NUP, DC_MEDIATYPES
def DeviceCapabilities(  # type: ignore[overload-overlap]
    Device: str, Port: str, Capability: Literal[2, 6, 33, 35], DEVMODE: _win32typing.PyDEVMODEW | None = None, /
) -> tuple[int, ...]: ...
@overload
# DC_BINNAMES, DC_FILEDEPENDENCIES, DC_PAPERNAMES, DC_PERSONALITY, DC_MEDIAREADY, DC_MEDIATYPENAMES
def DeviceCapabilities(  # type: ignore[overload-overlap]
    Device: str, Port: str, Capability: Literal[12, 14, 16, 25, 29, 34], DEVMODE: _win32typing.PyDEVMODEW | None = None, /
) -> tuple[str, ...]: ...
@overload
# DC_PAPERSIZE
def DeviceCapabilities(  # type: ignore[overload-overlap]
    Device: str, Port: str, Capability: Literal[3], DEVMODE: _win32typing.PyDEVMODEW | None = None, /
) -> tuple[_win32typing.PrinterPaperSize, ...]: ...
@overload
# DC_FIELDS, DC_DUPLEX, DC_SIZE, DC_EXTRA, DC_VERSION, DC_DRIVER, DC_TRUETYPE, DC_ORIENTATION, DC_COPIES
# DC_COLLATE, DC_PRINTRATE, DC_PRINTRATEUNIT, DC_PRINTERMEM, DC_STAPLE, DC_PRINTRATEPPM, DC_COLORDEVICE
def DeviceCapabilities(  # type: ignore[overload-overlap]
    Device: str,
    Port: str,
    Capability: Literal[1, 7, 8, 9, 10, 11, 15, 17, 18, 22, 26, 27, 28, 30, 31, 32],
    DEVMODE: _win32typing.PyDEVMODEW | None = None,
    /,
) -> int: ...
@overload
def DeviceCapabilities(Device: str, Port: str, Capability: int, DEVMODE: _win32typing.PyDEVMODEW | None = None, /) -> int: ...

def GetDeviceCaps(hdc: int | _win32typing.PyHANDLE, Index: int, /) -> int: ...

@overload
def EnumMonitors(Name: str | None, Level: Literal[1], /) -> tuple[_win32typing.MonitorInfo1, ...]: ...
@overload
def EnumMonitors(Name: str | None, Level: Literal[2], /) -> tuple[_win32typing.MonitorInfo2, ...]: ...

@overload
def EnumPorts(Name: str | None, Level: Literal[1], /) -> tuple[_win32typing.PortInfo1, ...]: ...
@overload
def EnumPorts(Name: str | None, Level: Literal[2], /) -> tuple[_win32typing.PortInfo2, ...]: ...

def GetPrintProcessorDirectory(Name: str | None = None, Environment: str | None = None, /) -> str: ...
def GetPrinterDriverDirectory(Name: str | None = None, Environment: str | None = None, /) -> str: ...
def AddPrinter(Name: str | None, Level: Literal[2], pPrinter: _win32typing.PrinterInfo2, /) -> _win32typing.PyPrinterHANDLE: ...
def DeletePrinter(hPrinter: _win32typing.PyPrinterHANDLE, /) -> None: ...
def DeletePrinterDriver(Server: str | None, Environment: str | None, DriverName: str, /) -> None: ...
def DeletePrinterDriverEx(
    Server: str | None, Environment: str | None, DriverName: str, DeleteFlag: int, VersionFlag: Literal[0, 1, 2, 3], /
) -> None: ...
def FlushPrinter(Printer: _win32typing.PyPrinterHANDLE, Buf: bytes, Sleep: int, /) -> int: ...

DEF_PRIORITY: int
DI_APPBANDING: int
DI_ROPS_READ_DESTINATION: int
DPD_DELETE_ALL_FILES: int
DPD_DELETE_SPECIFIC_VERSION: int
DPD_DELETE_UNUSED_FILES: int
DSPRINT_PENDING: int
DSPRINT_PUBLISH: int
DSPRINT_REPUBLISH: int
DSPRINT_UNPUBLISH: int
DSPRINT_UPDATE: int
FORM_BUILTIN: int
FORM_PRINTER: int
FORM_USER: int
JOB_ACCESS_ADMINISTER: int
JOB_ACCESS_READ: int
JOB_ALL_ACCESS: int
JOB_CONTROL_CANCEL: int
JOB_CONTROL_DELETE: int
JOB_CONTROL_LAST_PAGE_EJECTED: int
JOB_CONTROL_PAUSE: int
JOB_CONTROL_RESTART: int
JOB_CONTROL_RESUME: int
JOB_CONTROL_SENT_TO_PRINTER: int
JOB_EXECUTE: int
JOB_INFO_1: int
JOB_POSITION_UNSPECIFIED: int
JOB_READ: int
JOB_STATUS_BLOCKED_DEVQ: int
JOB_STATUS_COMPLETE: int
JOB_STATUS_DELETED: int
JOB_STATUS_DELETING: int
JOB_STATUS_ERROR: int
JOB_STATUS_OFFLINE: int
JOB_STATUS_PAPEROUT: int
JOB_STATUS_PAUSED: int
JOB_STATUS_PRINTED: int
JOB_STATUS_PRINTING: int
JOB_STATUS_RESTART: int
JOB_STATUS_SPOOLING: int
JOB_STATUS_USER_INTERVENTION: int
JOB_WRITE: int
MAX_PRIORITY: int
MIN_PRIORITY: int
PORT_STATUS_DOOR_OPEN: int
PORT_STATUS_NO_TONER: int
PORT_STATUS_OFFLINE: int
PORT_STATUS_OUTPUT_BIN_FULL: int
PORT_STATUS_OUT_OF_MEMORY: int
PORT_STATUS_PAPER_JAM: int
PORT_STATUS_PAPER_OUT: int
PORT_STATUS_PAPER_PROBLEM: int
PORT_STATUS_POWER_SAVE: int
PORT_STATUS_TONER_LOW: int
PORT_STATUS_TYPE_ERROR: int
PORT_STATUS_TYPE_INFO: int
PORT_STATUS_TYPE_WARNING: int
PORT_STATUS_USER_INTERVENTION: int
PORT_STATUS_WARMING_UP: int
PORT_TYPE_NET_ATTACHED: int
PORT_TYPE_READ: int
PORT_TYPE_REDIRECTED: int
PORT_TYPE_WRITE: int
PRINTER_ACCESS_ADMINISTER: int
PRINTER_ACCESS_USE: int
PRINTER_ALL_ACCESS: int
PRINTER_ATTRIBUTE_DEFAULT: int
PRINTER_ATTRIBUTE_DIRECT: int
PRINTER_ATTRIBUTE_DO_COMPLETE_FIRST: int
PRINTER_ATTRIBUTE_ENABLE_BIDI: int
PRINTER_ATTRIBUTE_ENABLE_DEVQ: int
PRINTER_ATTRIBUTE_FAX: int
PRINTER_ATTRIBUTE_HIDDEN: int
PRINTER_ATTRIBUTE_KEEPPRINTEDJOBS: int
PRINTER_ATTRIBUTE_LOCAL: int
PRINTER_ATTRIBUTE_NETWORK: int
PRINTER_ATTRIBUTE_PUBLISHED: int
PRINTER_ATTRIBUTE_QUEUED: int
PRINTER_ATTRIBUTE_RAW_ONLY: int
PRINTER_ATTRIBUTE_SHARED: int
PRINTER_ATTRIBUTE_TS: int
PRINTER_ATTRIBUTE_WORK_OFFLINE: int
PRINTER_CONTROL_PAUSE: int
PRINTER_CONTROL_PURGE: int
PRINTER_CONTROL_RESUME: int
PRINTER_CONTROL_SET_STATUS: int
PRINTER_ENUM_CONNECTIONS: int
PRINTER_ENUM_CONTAINER: int
PRINTER_ENUM_DEFAULT: int
PRINTER_ENUM_EXPAND: int
PRINTER_ENUM_ICON1: int
PRINTER_ENUM_ICON2: int
PRINTER_ENUM_ICON3: int
PRINTER_ENUM_ICON4: int
PRINTER_ENUM_ICON5: int
PRINTER_ENUM_ICON6: int
PRINTER_ENUM_ICON7: int
PRINTER_ENUM_ICON8: int
PRINTER_ENUM_LOCAL: int
PRINTER_ENUM_NAME: int
PRINTER_ENUM_NETWORK: int
PRINTER_ENUM_REMOTE: int
PRINTER_ENUM_SHARED: int
PRINTER_EXECUTE: int
PRINTER_INFO_1: Final = 1
PRINTER_READ: int
PRINTER_STATUS_BUSY: int
PRINTER_STATUS_DOOR_OPEN: int
PRINTER_STATUS_ERROR: int
PRINTER_STATUS_INITIALIZING: int
PRINTER_STATUS_IO_ACTIVE: int
PRINTER_STATUS_MANUAL_FEED: int
PRINTER_STATUS_NOT_AVAILABLE: int
PRINTER_STATUS_NO_TONER: int
PRINTER_STATUS_OFFLINE: int
PRINTER_STATUS_OUTPUT_BIN_FULL: int
PRINTER_STATUS_OUT_OF_MEMORY: int
PRINTER_STATUS_PAGE_PUNT: int
PRINTER_STATUS_PAPER_JAM: int
PRINTER_STATUS_PAPER_OUT: int
PRINTER_STATUS_PAPER_PROBLEM: int
PRINTER_STATUS_PAUSED: int
PRINTER_STATUS_PENDING_DELETION: int
PRINTER_STATUS_POWER_SAVE: int
PRINTER_STATUS_PRINTING: int
PRINTER_STATUS_PROCESSING: int
PRINTER_STATUS_SERVER_UNKNOWN: int
PRINTER_STATUS_TONER_LOW: int
PRINTER_STATUS_USER_INTERVENTION: int
PRINTER_STATUS_WAITING: int
PRINTER_STATUS_WARMING_UP: int
PRINTER_WRITE: int
SERVER_ACCESS_ADMINISTER: int
SERVER_ACCESS_ENUMERATE: int
SERVER_ALL_ACCESS: int
SERVER_EXECUTE: int
SERVER_READ: int
SERVER_WRITE: int



import re
from _typeshed.wsgi import WSGIApplication
from collections.abc import Callable, Mapping, Sequence
from typing import Any, Literal, TypeAlias, TypedDict, overload, type_check_only
from typing_extensions import Unpack
from xml.etree import ElementTree

from bs4 import BeautifulSoup
from webob import Response
from webtest.app import TestApp, TestRequest, _Files
from webtest.forms import Form

_Pattern: TypeAlias = str | bytes | re.Pattern[str] | Callable[[str], bool]
# NOTE: These are optional dependencies, so we don't want to depend on them
#       in the stubs either. Also there are no stubs for pyquery anyways.
_PyQuery: TypeAlias = Any
_PyQueryParams: TypeAlias = Any
_LxmlElement: TypeAlias = Any

@type_check_only
class _GetParams(TypedDict, total=False):
    params: Mapping[str, str] | str
    headers: Mapping[str, str]
    extra_environ: Mapping[str, Any]
    status: int | str | None
    expect_errors: bool
    xhr: bool

@type_check_only
class _PostParams(_GetParams, total=False):
    upload_files: _Files
    content_type: str

class TestResponse(Response):
    # NOTE: The way WebTest creates reponses the request is always set
    #       we could've used `MaybeNone`, but it seems more pragmatic
    #       to just assume that this is always set.
    request: TestRequest  # type: ignore[assignment]
    app: WSGIApplication
    test_app: TestApp
    parser_features: str | Sequence[str]
    __test__: Literal[False]
    @property
    def forms(self) -> dict[str | int, Form]: ...
    @property
    def form(self) -> Form: ...
    @property
    def testbody(self) -> str: ...
    def follow(self, **kw: Unpack[_GetParams]) -> TestResponse: ...
    def maybe_follow(self, **kw: Unpack[_GetParams]) -> TestResponse: ...
    def click(
        self,
        description: _Pattern | None = None,
        linkid: _Pattern | None = None,
        href: _Pattern | None = None,
        index: int | None = None,
        verbose: bool = False,
        extra_environ: dict[str, Any] | None = None,
    ) -> TestResponse: ...
    def clickbutton(
        self,
        description: _Pattern | None = None,
        buttonid: _Pattern | None = None,
        href: _Pattern | None = None,
        onclick: str | None = None,
        index: int | None = None,
        verbose: bool = False,
    ) -> TestResponse: ...

    @overload
    def goto(self, href: str, method: Literal["get"] = "get", **args: Unpack[_GetParams]) -> TestResponse: ...
    @overload
    def goto(self, href: str, method: Literal["post"], **args: Unpack[_PostParams]) -> TestResponse: ...

    @property
    def normal_body(self) -> bytes: ...
    @property
    def unicode_normal_body(self) -> str: ...
    def __contains__(self, s: str) -> bool: ...
    def mustcontain(self, *strings: str, no: Sequence[str] | str = ...) -> None: ...
    @property
    def html(self) -> BeautifulSoup: ...
    @property
    def xml(self) -> ElementTree.Element: ...
    @property
    def lxml(self) -> _LxmlElement: ...
    @property
    def json(self) -> Any: ...
    @property
    def pyquery(self) -> _PyQuery: ...
    def PyQuery(self, **kwargs: _PyQueryParams) -> _PyQuery: ...
    def showbrowser(self) -> None: ...
    def __str__(self) -> str: ...  # type: ignore[override]  # noqa: Y029



from _typeshed import Incomplete
from typing import Literal

unix_socket_available: bool

class Server:
    ipc: bool
    host: Incomplete
    port: Incomplete
    allowed_referral_hosts: Incomplete
    ssl: Incomplete
    tls: Incomplete
    name: Incomplete
    get_info: Incomplete
    dit_lock: Incomplete
    custom_formatter: Incomplete
    custom_validator: Incomplete
    current_address: Incomplete
    connect_timeout: Incomplete
    mode: Incomplete
    def __init__(
        self,
        host: str,
        port: int | None = None,
        use_ssl: bool = False,
        allowed_referral_hosts=None,
        get_info: Literal["NO_INFO", "DSA", "SCHEMA", "ALL"] = "SCHEMA",
        tls=None,
        formatter=None,
        connect_timeout=None,
        mode: Literal["IP_SYSTEM_DEFAULT", "IP_V4_ONLY", "IP_V6_ONLY", "IP_V4_PREFERRED", "IP_V6_PREFERRED"] = "IP_V6_PREFERRED",
        validator=None,
    ) -> None: ...
    @property
    def address_info(self): ...
    def update_availability(self, address, available) -> None: ...
    def reset_availability(self) -> None: ...
    def check_availability(self, source_address=None, source_port=None, source_port_list=None): ...
    @staticmethod
    def next_message_id(): ...
    def get_info_from_server(self, connection) -> None: ...
    def attach_dsa_info(self, dsa_info=None) -> None: ...
    def attach_schema_info(self, dsa_schema=None) -> None: ...
    @property
    def info(self): ...
    @property
    def schema(self): ...
    @staticmethod
    def from_definition(host, dsa_info, dsa_schema, port=None, use_ssl: bool = False, formatter=None, validator=None): ...
    def candidate_addresses(self): ...
    def has_control(self, control): ...
    def has_extension(self, extension): ...
    def has_feature(self, feature): ...



from _typeshed import Incomplete

from kafka.protocol.api import Request, Response

class AddPartitionsToTxnResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class AddPartitionsToTxnResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class AddPartitionsToTxnResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class AddPartitionsToTxnRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = AddPartitionsToTxnResponse_v0
    SCHEMA: Incomplete

class AddPartitionsToTxnRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = AddPartitionsToTxnResponse_v1
    SCHEMA: Incomplete

class AddPartitionsToTxnRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = AddPartitionsToTxnResponse_v2
    SCHEMA: Incomplete

AddPartitionsToTxnRequest: Incomplete
AddPartitionsToTxnResponse: Incomplete



from _typeshed import Incomplete
from _typeshed.wsgi import StartResponse, WSGIApplication, WSGIEnvironment
from collections.abc import Iterable
from threading import Thread
from typing import Literal, TypeAlias
from typing_extensions import Self

from waitress.server import TcpWSGIServer

# NOTE: We may never really be able to complete this, since `create`
#       invokes `cls.__init__` which is exempt from LSP violations
#       unless we get something like `KwArgsOf[cls.__init__]`.
_WSGIServerParams: TypeAlias = Incomplete

def get_free_port() -> tuple[str, int]: ...
def check_server(host: str, port: int, path_info: str = "/", timeout: float = 3, retries: int = 30) -> int: ...

class StopableWSGIServer(TcpWSGIServer):
    was_shutdown: bool
    runner: Thread
    test_app: WSGIApplication
    application_url: str
    def wrapper(self, environ: WSGIEnvironment, start_response: StartResponse) -> Iterable[bytes]: ...
    def run(self) -> None: ...
    def shutdown(self, debug: bool = False) -> Literal[True]: ...
    # NOTE: This has the same keyword arguments as cls.__init__, which
    #       we can't express
    @classmethod
    def create(cls, application: WSGIApplication, **kwargs: _WSGIServerParams) -> Self: ...
    def wait(self, retries: int = 30) -> bool: ...



from re import Pattern
from typing import Final, TypeAlias, overload

always_safe: Final[str]
urlencoded: Final[set[str]]
INVALID_HEX_PATTERN: Final[Pattern[str]]

_ExplodedQueryString: TypeAlias = list[tuple[str, str]]

def url_encode(params: _ExplodedQueryString) -> str: ...
def url_decode(query: str) -> _ExplodedQueryString: ...
def add_params_to_qs(query: str, params: _ExplodedQueryString | dict[str, str]) -> str: ...
def add_params_to_uri(uri: str, params: _ExplodedQueryString, fragment: bool = False) -> str: ...
def quote(s: str, safe: bytes = b"/") -> str: ...
def unquote(s: str | bytes) -> str: ...
def quote_url(s: str) -> str: ...

@overload
def extract_params(raw: None) -> None: ...
@overload
def extract_params(raw: dict[str, str]) -> _ExplodedQueryString: ...
@overload
def extract_params(raw: _ExplodedQueryString | tuple[tuple[str, str], ...] | str) -> _ExplodedQueryString | None: ...

def is_valid_url(url: str, fragments_allowed: bool = True) -> bool: ...



from _typeshed import Incomplete

from google.cloud.ndb import model

class BaseStatistic(model.Model):
    STORED_KIND_NAME: str
    bytes: Incomplete
    count: Incomplete
    timestamp: Incomplete

class BaseKindStatistic(BaseStatistic):
    STORED_KIND_NAME: str
    kind_name: Incomplete
    entity_bytes: Incomplete

class GlobalStat(BaseStatistic):
    STORED_KIND_NAME: str
    entity_bytes: Incomplete
    builtin_index_bytes: Incomplete
    builtin_index_count: Incomplete
    composite_index_bytes: Incomplete
    composite_index_count: Incomplete

class NamespaceStat(BaseStatistic):
    STORED_KIND_NAME: str
    subject_namespace: Incomplete
    entity_bytes: Incomplete
    builtin_index_bytes: Incomplete
    builtin_index_count: Incomplete
    composite_index_bytes: Incomplete
    composite_index_count: Incomplete

class KindStat(BaseKindStatistic):
    STORED_KIND_NAME: str
    builtin_index_bytes: Incomplete
    builtin_index_count: Incomplete
    composite_index_bytes: Incomplete
    composite_index_count: Incomplete

class KindRootEntityStat(BaseKindStatistic):
    STORED_KIND_NAME: str

class KindNonRootEntityStat(BaseKindStatistic):
    STORED_KIND_NAME: str

class PropertyTypeStat(BaseStatistic):
    STORED_KIND_NAME: str
    property_type: Incomplete
    entity_bytes: Incomplete
    builtin_index_bytes: Incomplete
    builtin_index_count: Incomplete

class KindPropertyTypeStat(BaseKindStatistic):
    STORED_KIND_NAME: str
    property_type: Incomplete
    builtin_index_bytes: Incomplete
    builtin_index_count: Incomplete

class KindPropertyNameStat(BaseKindStatistic):
    STORED_KIND_NAME: str
    property_name: Incomplete
    builtin_index_bytes: Incomplete
    builtin_index_count: Incomplete

class KindPropertyNamePropertyTypeStat(BaseKindStatistic):
    STORED_KIND_NAME: str
    property_type: Incomplete
    property_name: Incomplete
    builtin_index_bytes: Incomplete
    builtin_index_count: Incomplete

class KindCompositeIndexStat(BaseStatistic):
    STORED_KIND_NAME: str
    index_id: Incomplete
    kind_name: Incomplete

class NamespaceGlobalStat(GlobalStat):
    STORED_KIND_NAME: str

class NamespaceKindStat(KindStat):
    STORED_KIND_NAME: str

class NamespaceKindRootEntityStat(KindRootEntityStat):
    STORED_KIND_NAME: str

class NamespaceKindNonRootEntityStat(KindNonRootEntityStat):
    STORED_KIND_NAME: str

class NamespacePropertyTypeStat(PropertyTypeStat):
    STORED_KIND_NAME: str

class NamespaceKindPropertyTypeStat(KindPropertyTypeStat):
    STORED_KIND_NAME: str

class NamespaceKindPropertyNameStat(KindPropertyNameStat):
    STORED_KIND_NAME: str

class NamespaceKindPropertyNamePropertyTypeStat(KindPropertyNamePropertyTypeStat):
    STORED_KIND_NAME: str

class NamespaceKindCompositeIndexStat(KindCompositeIndexStat):
    STORED_KIND_NAME: str



from _typeshed import Incomplete

from kafka.sasl.abc import SaslMechanism

log: Incomplete

def xor_bytes(left, right): ...

class SaslMechanismScram(SaslMechanism):
    username: Incomplete
    mechanism: Incomplete
    def __init__(self, **config) -> None: ...
    def auth_bytes(self): ...
    def receive(self, auth_bytes): ...
    def is_done(self): ...
    def is_authenticated(self): ...
    def auth_details(self): ...

class ScramClient:
    MECHANISMS: Incomplete
    nonce: Incomplete
    auth_message: bytes
    salted_password: Incomplete
    user: Incomplete
    password: Incomplete
    hashfunc: Incomplete
    hashname: Incomplete
    stored_key: Incomplete
    client_key: Incomplete
    client_signature: Incomplete
    client_proof: Incomplete
    server_key: Incomplete
    server_signature: Incomplete
    def __init__(self, user, password, mechanism) -> None: ...
    def first_message(self): ...
    def process_server_first_message(self, server_first_message) -> None: ...
    def hmac(self, key, msg): ...
    def create_salted_password(self, salt, iterations) -> None: ...
    def final_message(self): ...
    def process_server_final_message(self, server_final_message) -> None: ...



from _typeshed import ReadableBuffer
from collections.abc import Iterable
from typing import SupportsBytes, SupportsIndex

def create_half_hash(
    s: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer, alg: str
) -> bytes | None: ...



import socket
import sys
from abc import ABCMeta
from re import Pattern
from typing import Final, SupportsIndex

RE_NUM: Final[Pattern[str]]
ON_LINUX: Final[bool]
ON_OSX: Final[bool]
ON_WINDOWS: Final[bool]

class AbstractBase(metaclass=ABCMeta): ...

SOCKET_ERROR = OSError
SOL_TCP: Final[int]
HAVE_SIGNAL: Final[bool]
str_or_bytes: Final[tuple[type[str], type[bytes]]]

def time_now() -> float: ...
def byte(*args: SupportsIndex) -> bytes: ...

class long(int): ...

def as_bytes(value: str | bytes) -> bytes: ...
def to_digit(value: str) -> int: ...
def get_linux_version(release_str: str) -> tuple[int, int, int]: ...

if sys.platform == "linux":
    LINUX_VERSION: Final[tuple[int, int, int]]
else:
    LINUX_VERSION: Final[None]

def nonblocking_socketpair(
    family: int = socket.AF_INET, socket_type: int = socket.SOCK_STREAM, proto: int = 0
) -> tuple[socket.socket, socket.socket]: ...



import datetime
from _typeshed import MaybeNone
from collections import OrderedDict
from collections.abc import Mapping
from logging import Logger
from typing import Any, TypeVar

from dateparser.conf import Settings

_DateT = TypeVar("_DateT", bound=datetime.date)
_DateTimeT = TypeVar("_DateTimeT", bound=datetime.datetime)

def strip_braces(date_string: str) -> str: ...
def normalize_unicode(string: str, form: str = "NFKD") -> str: ...
def combine_dicts(
    primary_dict: Mapping[Any, Any], supplementary_dict: Mapping[Any, Any]
) -> OrderedDict[str, str | list[Any]]: ...
def find_date_separator(format: str) -> str | MaybeNone | None: ...
def localize_timezone(date_time: _DateTimeT, tz_string: str) -> _DateTimeT: ...
def apply_tzdatabase_timezone(date_time: _DateTimeT, pytz_string: str) -> _DateTimeT: ...
def apply_dateparser_timezone(utc_datetime: _DateTimeT, offset_or_timezone_abb) -> _DateTimeT | None: ...
def apply_timezone(date_time: _DateTimeT, tz_string: str) -> _DateTimeT: ...
def apply_timezone_from_settings(date_obj: _DateTimeT, settings: Settings | None) -> _DateTimeT: ...
def get_last_day_of_month(year: int, month: int) -> int: ...
def get_previous_leap_year(year: int) -> int: ...
def get_next_leap_year(year: int) -> int: ...
def set_correct_day_from_settings(date_obj: _DateT, settings: Settings, current_day: int | None = None) -> _DateT: ...
def set_correct_month_from_settings(date_obj: _DateT, settings: Settings, current_month: int | None = None) -> _DateT: ...
def registry(cls): ...
def get_logger() -> Logger: ...
def setup_logging() -> None: ...

# TODO: this needs `types-pytz` and a type-alias
def get_timezone_from_tz_string(tz_string: str): ...



import selectors
import ssl
from _typeshed import Incomplete
from collections.abc import Callable, Iterable, Mapping, Sequence
from typing import Literal, TypeAlias, TypedDict, type_check_only
from typing_extensions import Unpack

from kafka.admin.acl_resource import ACL, ACLFilter
from kafka.admin.config_resource import ConfigResource
from kafka.admin.new_partitions import NewPartitions
from kafka.admin.new_topic import NewTopic
from kafka.errors import KafkaError
from kafka.protocol.admin import ElectionType
from kafka.structs import GroupInformation, OffsetAndMetadata, TopicPartition

_ApiVersion: TypeAlias = tuple[int, ...]
_BootstrapServers: TypeAlias = str | Sequence[str]
_KafkaClientFactory: TypeAlias = Callable[..., object]
_SaslMechanism: TypeAlias = Literal["PLAIN", "GSSAPI", "OAUTHBEARER", "SCRAM-SHA-256", "SCRAM-SHA-512"]
_SecurityProtocol: TypeAlias = Literal["PLAINTEXT", "SSL", "SASL_PLAINTEXT", "SASL_SSL"]
_SocketOption: TypeAlias = tuple[int, int, int]

@type_check_only
class _KafkaAdminClientConfig(TypedDict, total=False):
    bootstrap_servers: _BootstrapServers
    client_id: str
    request_timeout_ms: int
    connections_max_idle_ms: int
    reconnect_backoff_ms: int
    reconnect_backoff_max_ms: int
    max_in_flight_requests_per_connection: int
    receive_buffer_bytes: int | None
    send_buffer_bytes: int | None
    socket_options: Sequence[_SocketOption]
    sock_chunk_bytes: int
    sock_chunk_buffer_count: int
    retry_backoff_ms: int
    metadata_max_age_ms: int
    security_protocol: _SecurityProtocol
    ssl_context: ssl.SSLContext | None
    ssl_check_hostname: bool
    ssl_cafile: str | None
    ssl_certfile: str | None
    ssl_keyfile: str | None
    ssl_password: str | None
    ssl_crlfile: str | None
    api_version: _ApiVersion | None
    api_version_auto_timeout_ms: int
    selector: type[selectors.BaseSelector]
    sasl_mechanism: _SaslMechanism | None
    sasl_plain_username: str | None
    sasl_plain_password: str | None
    sasl_kerberos_name: object | None
    sasl_kerberos_service_name: str
    sasl_kerberos_domain_name: str | None
    sasl_oauth_token_provider: object | None
    socks5_proxy: str | None
    metric_reporters: Sequence[type[object]]
    metrics_num_samples: int
    metrics_sample_window_ms: int
    kafka_client: _KafkaClientFactory

@type_check_only
class _CreateAclsResult(TypedDict):
    succeeded: list[ACL]
    failed: list[tuple[ACL, KafkaError]]

log: Incomplete

class KafkaAdminClient:
    DEFAULT_CONFIG: Incomplete
    config: Incomplete
    def __init__(self, **configs: Unpack[_KafkaAdminClientConfig]) -> None: ...
    def close(self) -> None: ...
    def send_request(self, request, node_id=None): ...
    def send_requests(self, requests_and_node_ids, response_fn=...): ...
    def create_topics(self, new_topics: Sequence[NewTopic], timeout_ms: int | None = None, validate_only: bool = False): ...
    def delete_topics(self, topics: Sequence[str], timeout_ms: int | None = None): ...
    def list_topics(self) -> list[str]: ...
    def describe_topics(self, topics: Sequence[str] | None = None) -> list[dict[str, Incomplete]]: ...
    def describe_cluster(self) -> dict[str, Incomplete]: ...
    def describe_acls(self, acl_filter: ACLFilter) -> tuple[list[ACL], KafkaError]: ...
    def create_acls(self, acls: Sequence[ACL]) -> _CreateAclsResult: ...
    def delete_acls(
        self, acl_filters: Sequence[ACLFilter]
    ) -> list[tuple[ACLFilter, list[tuple[ACL, KafkaError]], KafkaError]]: ...
    def describe_configs(self, config_resources: Sequence[ConfigResource], include_synonyms: bool = False): ...
    def alter_configs(self, config_resources: Sequence[ConfigResource]): ...
    def create_partitions(
        self, topic_partitions: Mapping[str, NewPartitions], timeout_ms: int | None = None, validate_only: bool = False
    ): ...
    def delete_records(
        self,
        records_to_delete: Mapping[TopicPartition, int],
        timeout_ms: float | None = None,
        partition_leader_id: int | None = None,
    ) -> dict[TopicPartition, Incomplete]: ...
    def describe_consumer_groups(
        self, group_ids: Sequence[str], group_coordinator_id: int | None = None, include_authorized_operations: bool = False
    ) -> list[GroupInformation]: ...
    def list_consumer_groups(self, broker_ids: Sequence[int] | None = None) -> list[tuple[str, str]]: ...
    def list_consumer_group_offsets(
        self, group_id: str, group_coordinator_id: int | None = None, partitions: Iterable[TopicPartition] | None = None
    ) -> dict[TopicPartition, OffsetAndMetadata]: ...
    def delete_consumer_groups(
        self, group_ids: Sequence[str], group_coordinator_id: int | None = None
    ) -> list[tuple[str, KafkaError]]: ...
    def perform_leader_election(
        self,
        election_type: int | ElectionType,
        topic_partitions: Mapping[str, Sequence[int]] | None = None,
        timeout_ms: int | None = None,
    ): ...
    def describe_log_dirs(self): ...



from _typeshed import Incomplete
from collections.abc import Callable, Iterable, Sequence
from re import Match, Pattern
from types import ModuleType, SimpleNamespace as Struct
from typing import Any, ClassVar, Final, NoReturn, TypeAlias

from docutils import ApplicationError, DataError, nodes
from docutils.nodes import Node, system_message
from docutils.parsers.rst.languages import _RstLanguageModule
from docutils.statemachine import StateMachine, StateMachineWS, StateWS, StringList
from docutils.utils import Reporter

__docformat__: Final = "reStructuredText"

class MarkupError(DataError): ...
class UnknownInterpretedRoleError(DataError): ...
class InterpretedRoleNotImplementedError(DataError): ...
class ParserError(ApplicationError): ...
class MarkupMismatch(Exception): ...

class RSTStateMachine(StateMachineWS[list[str]]):
    language: _RstLanguageModule
    match_titles: bool
    memo: Struct | None
    document: nodes.document
    reporter: Reporter
    node: nodes.document | None
    section_level_offset: int
    def run(  # type: ignore[override]
        self,
        input_lines: Sequence[str] | StringList,
        document: nodes.document,
        input_offset: int = 0,
        match_titles: bool = True,
        inliner: Inliner | None = None,
    ) -> None: ...

class NestedStateMachine(RSTStateMachine):
    parent_state_machine: Incomplete | None
    def __init__(self, state_classes, initial_state, debug: bool = False, parent_state_machine=None) -> None: ...
    def run(  # type: ignore[override]
        self, input_lines: Sequence[str] | StringList, input_offset: int, memo, node, match_titles: bool = True
    ) -> list[str]: ...

class RSTState(StateWS[list[str]]):
    nested_sm: type[NestedStateMachine]
    nested_sm_cache: list[StateMachine[Incomplete]]
    def __init__(self, state_machine, debug: bool = False) -> None: ...
    memo: Incomplete
    reporter: Reporter
    inliner: Inliner
    document: nodes.document
    parent: Incomplete
    def runtime_init(self) -> None: ...
    def goto_line(self, abs_line_offset: int) -> None: ...
    def no_match(self, context: list[str], transitions): ...
    def bof(self, context: list[str]): ...
    def nested_parse(
        self,
        block: StringList,
        input_offset: int,
        node: nodes.Element | None = None,
        match_titles: bool = False,
        state_machine_class: StateMachineWS[Incomplete] | None = None,
        state_machine_kwargs: dict[Incomplete, Incomplete] | None = None,
    ) -> int: ...
    def nested_list_parse(
        self,
        block,
        input_offset: int,
        node,
        initial_state,
        blank_finish,
        blank_finish_state=None,
        extra_settings={},
        match_titles: bool = False,
        state_machine_class=None,
        state_machine_kwargs=None,
    ): ...
    def section(self, title: str, source, style, lineno: int, messages) -> None: ...
    def check_subsection(self, source, style, lineno: int): ...
    def title_inconsistent(self, sourcetext: str, lineno: int): ...
    def new_subsection(self, title: str, lineno: int, messages) -> None: ...
    def paragraph(self, lines: Iterable[str], lineno: int): ...
    def inline_text(self, text: str, lineno: int) -> tuple[list[Node], list[system_message]]: ...
    def unindent_warning(self, node_name: str): ...

def build_regexp(definition, compile_patterns: bool | None = True): ...

_BasicDefinition: TypeAlias = tuple[str, str, str, list[Pattern[str]]]
_DefinitionParts: TypeAlias = tuple[str, str, str, list[Pattern[str] | _BasicDefinition]]
_DefinitionType: TypeAlias = tuple[str, str, str, list[Pattern[str] | _DefinitionParts]]

class Inliner:
    implicit_dispatch: list[tuple[Pattern[str], Callable[[Match[str], int], Sequence[nodes.Node]]]]
    def __init__(self) -> None: ...
    start_string_prefix: str
    end_string_suffix: str
    parts: _DefinitionType
    patterns: Struct
    def init_customizations(self, settings: Any) -> None: ...
    reporter: Reporter
    document: nodes.document
    language: ModuleType
    parent: nodes.Element
    def parse(
        self, text: str, lineno: int, memo: Struct, parent: nodes.Element
    ) -> tuple[list[nodes.Node], list[nodes.system_message]]: ...
    non_whitespace_before: str
    non_whitespace_escape_before: str
    non_unescaped_whitespace_escape_before: str
    non_whitespace_after: str
    simplename: str
    uric: str
    uri_end_delim: str
    urilast: str
    uri_end: str
    emailc: str
    email_pattern: str
    def quoted_start(self, match: Match[str]) -> bool: ...
    def inline_obj(
        self,
        match: Match[str],
        lineno: int,
        end_pattern: Pattern[str],
        nodeclass: nodes.TextElement,
        restore_backslashes: bool = False,
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message], str]: ...
    def problematic(self, text: str, rawsource: str, message: nodes.system_message) -> nodes.problematic: ...
    def emphasis(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def strong(self, match: Match[str], lineno: int) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def interpreted_or_phrase_ref(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def phrase_ref(
        self, before: str, after: str, rawsource: str, escaped: str, text: str | None = None
    ) -> tuple[str, list[nodes.Node], str, list[nodes.Node]]: ...
    def adjust_uri(self, uri: str) -> str: ...
    def interpreted(
        self, rawsource: str, text: str, role: str, lineno: int
    ) -> tuple[list[nodes.Node], list[nodes.system_message]]: ...
    def literal(self, match: Match[str], lineno: int) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def inline_internal_target(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def substitution_reference(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def footnote_reference(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def reference(
        self, match: Match[str], lineno: int, anonymous: bool = False
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def anonymous_reference(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def standalone_uri(
        self, match: Match[str], lineno: int
    ) -> list[tuple[str, list[nodes.problematic], str, list[nodes.system_message]]]: ...
    def pep_reference(
        self, match: Match[str], lineno: int
    ) -> list[tuple[str, list[nodes.problematic], str, list[nodes.system_message]]]: ...
    rfc_url: str = ...
    def rfc_reference(
        self, match: Match[str], lineno: int
    ) -> list[tuple[str, list[nodes.problematic], str, list[nodes.system_message]]]: ...
    def implicit_inline(self, text: str, lineno: int) -> list[nodes.Text]: ...
    dispatch: dict[str, Callable[[Match[str], int], tuple[str, list[nodes.problematic], str, list[nodes.system_message]]]] = ...

class Body(RSTState):
    double_width_pad_char: Incomplete
    enum: Incomplete
    grid_table_top_pat: Incomplete
    simple_table_top_pat: Incomplete
    simple_table_border_pat: Incomplete
    pats: Incomplete
    patterns: ClassVar[dict[str, str | Pattern[str]]]
    initial_transitions: ClassVar[tuple[str, ...]]
    sequence: str
    format: str
    def indent(self, match, context, next_state): ...
    def block_quote(self, indented, line_offset): ...
    attribution_pattern: Incomplete
    def split_attribution(self, indented, line_offset): ...
    def check_attribution(self, indented, attribution_start): ...
    def parse_attribution(self, indented, line_offset): ...
    def bullet(self, match, context, next_state): ...
    def list_item(self, indent): ...
    def enumerator(self, match, context, next_state): ...
    def parse_enumerator(self, match, expected_sequence=None): ...
    def is_enumerated_list_item(self, ordinal, sequence, format): ...
    def make_enumerator(self, ordinal, sequence, format): ...
    def field_marker(self, match, context, next_state): ...
    def field(self, match): ...
    def parse_field_marker(self, match): ...
    def parse_field_body(self, indented, offset, node) -> None: ...
    def option_marker(self, match, context, next_state): ...
    def option_list_item(self, match): ...
    def parse_option_marker(self, match): ...
    def doctest(self, match, context, next_state): ...
    def line_block(self, match, context, next_state): ...
    def line_block_line(self, match, lineno): ...
    def nest_line_block_lines(self, block) -> None: ...
    def nest_line_block_segment(self, block) -> None: ...
    def grid_table_top(self, match, context, next_state): ...
    def simple_table_top(self, match, context, next_state): ...
    def table_top(self, match, context, next_state, isolate_function, parser_class): ...
    def table(self, isolate_function, parser_class): ...
    def isolate_grid_table(self): ...
    def isolate_simple_table(self): ...
    def malformed_table(self, block, detail: str = "", offset: int = 0): ...
    def build_table(self, tabledata, tableline, stub_columns: int = 0, widths=None) -> nodes.table: ...
    def build_table_row(self, rowdata, tableline): ...
    explicit: Incomplete
    def footnote(self, match): ...
    def citation(self, match): ...
    def hyperlink_target(self, match): ...
    def make_target(self, block, block_text, lineno, target_name): ...
    def parse_target(self, block, block_text, lineno): ...
    def is_reference(self, reference): ...
    def add_target(self, targetname, refuri, target, lineno) -> None: ...
    def substitution_def(self, match): ...
    def disallowed_inside_substitution_definitions(self, node): ...
    def directive(self, match, **option_presets): ...
    def run_directive(self, directive, match, type_name, option_presets): ...
    def parse_directive_block(self, indented, line_offset, directive, option_presets): ...
    def parse_directive_options(self, option_presets, option_spec, arg_block): ...
    def parse_directive_arguments(self, directive, arg_block): ...
    def parse_extension_options(self, option_spec, datalines): ...
    def unknown_directive(self, type_name): ...
    def comment(self, match): ...
    def explicit_markup(self, match, context, next_state): ...
    def explicit_construct(self, match): ...
    def explicit_list(self, blank_finish) -> None: ...
    def anonymous(self, match: Match[str], context: list[str] | None, next_state: str): ...
    def anonymous_target(self, match): ...
    def line(self, match, context, next_state): ...
    def text(self, match, context, next_state): ...

class RFC2822Body(Body):
    patterns: ClassVar[dict[str, str | Pattern[str]]]
    initial_transitions: ClassVar[list[tuple[str | tuple[str, str], str]]]  # type: ignore[assignment]
    def rfc2822(self, match, context, next_state): ...
    def rfc2822_field(self, match): ...

class SpecializedBody(Body):
    def invalid_input(
        self, match: Match[str] | None = None, context: list[str] | None = None, next_state: str | None = None
    ) -> NoReturn: ...
    indent = invalid_input  # type: ignore[assignment]
    bullet = invalid_input
    enumerator = invalid_input
    field_marker = invalid_input
    option_marker = invalid_input
    doctest = invalid_input
    line_block = invalid_input
    grid_table_top = invalid_input
    simple_table_top = invalid_input
    explicit_markup = invalid_input
    anonymous = invalid_input  # type: ignore[assignment]
    line = invalid_input
    text = invalid_input

class BulletList(SpecializedBody):
    blank_finish: Incomplete
    def bullet(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...

class DefinitionList(SpecializedBody):
    def text(self, match: Match[str], context: list[str] | None, next_state: str | None) -> tuple[list[str], str, list[str]]: ...  # type: ignore[override]

class EnumeratedList(SpecializedBody):
    auto: int
    blank_finish: Incomplete
    lastordinal: Incomplete
    def enumerator(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...

class FieldList(SpecializedBody):
    blank_finish: Incomplete
    def field_marker(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...

class OptionList(SpecializedBody):
    blank_finish: Incomplete
    def option_marker(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...

class RFC2822List(SpecializedBody, RFC2822Body):
    patterns: ClassVar[dict[str, str | Pattern[str]]]
    initial_transitions: ClassVar[list[tuple[str | tuple[str, str], str]]]  # type: ignore[assignment]
    blank_finish: Incomplete
    def rfc2822(self, match, context, next_state): ...
    blank: Incomplete

class ExtensionOptions(FieldList):
    def parse_field_body(self, indented, offset, node) -> None: ...

class LineBlock(SpecializedBody):
    blank: Incomplete
    blank_finish: Incomplete
    def line_block(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...

class Explicit(SpecializedBody):
    blank_finish: Incomplete
    blank: Incomplete
    def explicit_markup(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...
    def anonymous(  # type: ignore[override]
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...

class SubstitutionDef(Body):
    patterns: ClassVar[dict[str, str | Pattern[str]]]
    initial_transitions: ClassVar[list[str]]  # type: ignore[assignment]
    blank_finish: Incomplete
    def embedded_directive(self, match, context, next_state) -> None: ...
    def text(self, match, context, next_state) -> None: ...

class Text(RSTState):
    patterns: ClassVar[dict[str, str | Pattern[str]]]
    initial_transitions: ClassVar[list[tuple[str, str]]]
    def blank(self, match, context, next_state): ...
    def eof(self, context): ...
    def indent(self, match, context, next_state): ...
    def underline(self, match, context, next_state): ...
    def text(self, match, context, next_state): ...
    def literal_block(self): ...
    def quoted_literal_block(self): ...
    def definition_list_item(self, termline): ...
    classifier_delimiter: Incomplete
    def term(self, lines, lineno): ...

class SpecializedText(Text):
    def eof(self, context): ...
    def invalid_input(
        self, match: Match[str] | None = None, context: list[str] | None = None, next_state: str | None = None
    ) -> NoReturn: ...
    blank = invalid_input
    indent = invalid_input
    underline = invalid_input
    text = invalid_input

class Definition(SpecializedText):
    def eof(self, context): ...
    blank_finish: Incomplete
    def indent(  # type: ignore[override]
        self, match: Match[str] | None, context: list[str], next_state: str | None
    ) -> tuple[list[str], str, list[str]]: ...

class Line(SpecializedText):
    eofcheck: int
    def eof(self, context: list[str]): ...
    def blank(self, match: Match[str] | None, context: list[str], next_state: str | None) -> tuple[list[str], str, list[str]]: ...  # type: ignore[override]
    def text(self, match: Match[str], context: list[str], next_state: str | None) -> tuple[list[str], str, list[str]]: ...  # type: ignore[override]
    indent = text  # type: ignore[assignment]
    def underline(  # type: ignore[override]
        self, match: Match[str] | None, context: list[str], next_state: str | None
    ) -> tuple[list[str], str, list[str]]: ...
    def short_overline(self, context, blocktext, lineno, lines: int = 1) -> None: ...
    def state_correction(self, context, lines: int = 1) -> None: ...

class QuotedLiteralBlock(RSTState):
    patterns: ClassVar[dict[str, str | Pattern[str]]]
    messages: Incomplete
    initial_lineno: Incomplete
    def __init__(self, state_machine, debug: bool = False) -> None: ...
    def blank(self, match, context, next_state): ...
    def eof(self, context): ...
    def indent(self, match: Match[str] | None, context: list[str], next_state: str | None) -> NoReturn: ...
    def initial_quoted(
        self, match: Match[str], context: list[str] | None, next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...
    def quoted(
        self, match: Match[str], context: list[str], next_state: str | None
    ) -> tuple[list[str], str | None, list[str]]: ...
    def text(self, match: Match[str] | None, context: list[str] | None, next_state: str | None) -> None: ...

state_classes: tuple[type[RSTState], ...]



from webtest.app import AppError as AppError, TestApp as TestApp, TestRequest as TestRequest
from webtest.forms import (
    Checkbox as Checkbox,
    Field as Field,
    Form as Form,
    Hidden as Hidden,
    Radio as Radio,
    Select as Select,
    Submit as Submit,
    Text as Text,
    Textarea as Textarea,
    Upload as Upload,
)
from webtest.response import TestResponse as TestResponse



import argparse
from _typeshed import ConvertibleToInt
from collections.abc import Callable, Container
from ssl import SSLContext, _SSLMethod
from typing import Annotated, Any, ClassVar, Final, TypeAlias, overload

from gunicorn.arbiter import Arbiter
from gunicorn.glogging import Logger as GLogger
from gunicorn.http import Request
from gunicorn.http.wsgi import Response
from gunicorn.workers.base import Worker

from ._types import _AddressType, _EnvironType

_ConfigValueType: TypeAlias = Any
# Any: Value type depends on the setting's validator (e.g., validate_bool, validate_pos_int).
# Maybe types: bool, int, str, list[str], dict[str, Any], Callable[..., object], type[object], ssl.PROTOCOL_*.
# Validator ensures type correctness at runtime.

_OnStartingHookType: TypeAlias = Callable[[Arbiter], object]
_OnReloadHookType: TypeAlias = Callable[[Arbiter], object]
_WhenReadyHookType: TypeAlias = Callable[[Arbiter], object]
_PreForkHookType: TypeAlias = Callable[[Arbiter, Worker], object]
_PostForkHookType: TypeAlias = Callable[[Arbiter, Worker], object]
_PostWorkerInitHookType: TypeAlias = Callable[[Worker], object]
_WorkerIntHookType: TypeAlias = Callable[[Worker], object]
_WorkerAbortHookType: TypeAlias = Callable[[Worker], object]
_PreExecHookType: TypeAlias = Callable[[Arbiter], object]
_PreRequestHookType: TypeAlias = Callable[[Worker, Request], object]
_PostRequestHookType: TypeAlias = Callable[[Worker, Request, _EnvironType, Response], object]
_ChildExitHookType: TypeAlias = Callable[[Arbiter, Worker], object]
_WorkerExitHookType: TypeAlias = Callable[[Arbiter, Worker], object]
_NumWorkersChangedHookType: TypeAlias = Callable[[Arbiter, int, int | None], object]
_OnExitHookType: TypeAlias = Callable[[Arbiter], object]
_SSLContextHookType: TypeAlias = Callable[[Config, Callable[[], SSLContext]], SSLContext]
_OnDirtyStartingHookType: TypeAlias = Callable[[Arbiter], object]
_DirtyPostForkHookType: TypeAlias = Callable[[Arbiter, Worker], object]
_DirtyWorkerInitHookType: TypeAlias = Callable[[Worker], object]
_DirtyWorkerExitHookType: TypeAlias = Callable[[Arbiter, Worker], object]

_HookType: TypeAlias = (
    _OnStartingHookType
    | _OnReloadHookType
    | _WhenReadyHookType
    | _PreForkHookType
    | _PostForkHookType
    | _PostWorkerInitHookType
    | _WorkerIntHookType
    | _WorkerAbortHookType
    | _PreExecHookType
    | _PreRequestHookType
    | _PostRequestHookType
    | _ChildExitHookType
    | _WorkerExitHookType
    | _NumWorkersChangedHookType
    | _OnExitHookType
    | _SSLContextHookType
    | _OnDirtyStartingHookType
    | _DirtyPostForkHookType
    | _DirtyWorkerInitHookType
    | _DirtyWorkerExitHookType
)
# Validators
_BoolValidatorType: TypeAlias = Callable[[bool | str | None], bool | None]
_StringValidatorType: TypeAlias = Callable[[str | None], str | None]
_ListStringValidatorType: TypeAlias = Callable[[str | list[str] | None], list[str]]
_IntValidatorType: TypeAlias = Callable[[ConvertibleToInt], int]
_DictValidatorType: TypeAlias = Callable[[dict[str, Any]], dict[str, Any]]
_ClassValidatorType: TypeAlias = Callable[[object | str | None], type[Any] | None]
_UserGroupValidatorType: TypeAlias = Callable[[str | int | None], int]
_AddressValidatorType: TypeAlias = Callable[[str | None], _AddressType | None]
_CallableValidatorType: TypeAlias = Callable[[str | _HookType], _HookType]
_ProxyProtocolValidatorType: TypeAlias = Callable[[str | bool | None], str]
_ASGILoopValidatorType: TypeAlias = Callable[[str | None], str]
_ASGILifespanValidatorType: TypeAlias = Callable[[str | None], str]
_HTTP2FrameSizeValidatorType: TypeAlias = Callable[[ConvertibleToInt], int]
_HTTPProtocolsValidatorType: TypeAlias = Callable[[str | None], list[str]]
_HttpParserValidatorType: TypeAlias = Callable[[str | None], str]

_ValidatorType: TypeAlias = (  # noqa: Y047
    _BoolValidatorType
    | _StringValidatorType
    | _ListStringValidatorType
    | _IntValidatorType
    | _DictValidatorType
    | _ClassValidatorType
    | _UserGroupValidatorType
    | _AddressValidatorType
    | _CallableValidatorType
    | _ProxyProtocolValidatorType
    | _ASGILoopValidatorType
    | _ASGILifespanValidatorType
    | _HTTP2FrameSizeValidatorType
    | _HTTPProtocolsValidatorType
    | _HttpParserValidatorType
)

KNOWN_SETTINGS: list[Setting]
PLATFORM: str

def make_settings(ignore: Container[Setting] | None = None) -> dict[str, Setting]: ...
def auto_int(_: Any, x: str) -> int: ...

class Config:
    settings: dict[str, Setting]
    usage: str | None
    prog: str | None
    env_orig: dict[str, str]

    def __init__(self, usage: str | None = None, prog: str | None = None) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...
    def set(self, name: str, value: _ConfigValueType) -> None: ...
    def get_cmd_args_from_env(self) -> list[str]: ...
    def parser(self) -> argparse.ArgumentParser: ...
    @property
    def worker_class_str(self) -> str: ...
    @property
    def worker_class(self) -> type[Worker]: ...
    @property
    def address(self) -> list[_AddressType]: ...
    @property
    def uid(self) -> int: ...
    @property
    def gid(self) -> int: ...
    @property
    def proc_name(self) -> str | None: ...
    @property
    def logger_class(self) -> type[GLogger]: ...
    @property
    def is_ssl(self) -> bool: ...
    @property
    def ssl_options(self) -> dict[str, Any]: ...
    @property
    def env(self) -> dict[str, str]: ...
    @property
    def sendfile(self) -> bool: ...
    @property
    def reuse_port(self) -> bool: ...
    @property
    def paste_global_conf(self) -> dict[str, str] | None: ...

class SettingMeta(type):
    def __new__(cls, name: str, bases: tuple[type, ...], attrs: dict[str, Any]) -> SettingMeta: ...
    def fmt_desc(cls, desc: str) -> None: ...

class Setting(metaclass=SettingMeta):
    name: ClassVar[str | None]
    value: _ConfigValueType
    section: ClassVar[str | None]
    cli: ClassVar[list[str] | None]
    validator: ClassVar[Callable[..., Any] | None]  # See `_ValidatorType`
    type: ClassVar[argparse._ActionType | None]
    meta: ClassVar[str | None]
    action: ClassVar[str | None]
    default: ClassVar[Any]
    short: ClassVar[str | None]
    desc: ClassVar[str | None]
    nargs: ClassVar[int | str | None]
    const: ClassVar[bool | str | None]
    order: ClassVar[int]

    def __init__(self) -> None: ...
    def add_option(self, parser: argparse.ArgumentParser) -> None: ...
    def copy(self) -> Setting: ...
    def get(self) -> _ConfigValueType: ...
    def set(self, val: _ConfigValueType) -> None: ...
    def __lt__(self, other: Setting) -> bool: ...

    __cmp__ = __lt__

@overload
def validate_bool(val: bool) -> bool: ...
@overload
def validate_bool(val: None) -> None: ...
@overload
def validate_bool(val: Annotated[str, "Case-insensitive boolean string ('true'/'false' in any case)"]) -> bool: ...

def validate_dict(val: dict[str, Any]) -> dict[str, Any]: ...
def validate_pos_int(val: ConvertibleToInt) -> int: ...
def validate_http2_frame_size(val: ConvertibleToInt) -> int: ...
def validate_ssl_version(val: _SSLMethod) -> _SSLMethod: ...

@overload
def validate_string(val: str) -> str: ...
@overload
def validate_string(val: None) -> None: ...

@overload
def validate_file_exists(val: str) -> str: ...
@overload
def validate_file_exists(val: None) -> None: ...

def validate_list_string(val: str | list[str] | None) -> list[str]: ...
def validate_list_of_existing_files(val: str | list[str] | None) -> list[str]: ...
def validate_string_to_addr_list(val: str | None) -> list[str]: ...
def validate_string_to_list(val: str | None) -> list[str]: ...

@overload
def validate_class(val: str) -> str: ...
@overload
def validate_class(val: None) -> None: ...
@overload
def validate_class(val: object) -> object: ...

def validate_callable(arity: int) -> _CallableValidatorType: ...
def validate_user(val: int | str | None) -> int: ...
def validate_group(val: int | str | None) -> int: ...
def validate_post_request(val: str | _HookType) -> _PostRequestHookType: ...
def validate_chdir(val: str) -> str: ...

@overload
def validate_statsd_address(val: str) -> _AddressType: ...
@overload
def validate_statsd_address(val: None) -> None: ...

def validate_reload_engine(val: str) -> str: ...

@overload
def validate_header_map_behaviour(val: str) -> str: ...
@overload
def validate_header_map_behaviour(val: None) -> None: ...

def get_default_config_file() -> str | None: ...

class ConfigFile(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class WSGIApp(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class Bind(Setting):
    name: ClassVar[str]
    action: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[list[str]]
    desc: ClassVar[str]

class Backlog(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class Workers(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class WorkerClass(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ClassValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class WorkerThreads(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class WorkerConnections(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class MaxRequests(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class MaxRequestsJitter(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class Timeout(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class GracefulTimeout(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class Keepalive(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class LimitRequestLine(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class LimitRequestFields(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class LimitRequestFieldSize(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class Reload(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class ReloadEngine(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[Callable[[str], str]]
    default: ClassVar[str]
    desc: ClassVar[str]

class ReloadExtraFiles(Setting):
    name: ClassVar[str]
    action: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[list[str]]
    desc: ClassVar[str]

class Spew(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class ConfigCheck(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class PrintConfig(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class PreloadApp(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class Sendfile(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    const: ClassVar[bool]
    desc: ClassVar[str]

class ReusePort(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class Chdir(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[Callable[[str], str]]
    default: ClassVar[str]
    default_doc: ClassVar[str]
    desc: ClassVar[str]

class Daemon(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class Env(Setting):
    name: ClassVar[str]
    action: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[list[str]]
    desc: ClassVar[str]

class Pidfile(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class WorkerTmpDir(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class User(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_UserGroupValidatorType]
    default: ClassVar[int]
    default_doc: ClassVar[str]
    desc: ClassVar[str]

class Group(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_UserGroupValidatorType]
    default: ClassVar[int]
    default_doc: ClassVar[str]
    desc: ClassVar[str]

class Umask(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[Callable[[Any, str], int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class Initgroups(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class TmpUploadDir(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class SecureSchemeHeader(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_DictValidatorType]
    default: ClassVar[dict[str, str]]
    desc: ClassVar[str]

class ForwardedAllowIPS(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class AccessLog(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class DisableRedirectAccessToSyslog(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class AccessLogFormat(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class ErrorLog(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class Loglevel(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class CaptureOutput(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class LoggerClass(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ClassValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class LogConfig(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class LogConfigDict(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_DictValidatorType]
    default: ClassVar[dict[str, Any]]
    desc: ClassVar[str]

class LogConfigJson(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class SyslogTo(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]
    default_doc: ClassVar[str]

class Syslog(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class SyslogPrefix(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class SyslogFacility(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class EnableStdioInheritance(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    default: ClassVar[bool]
    action: ClassVar[str]
    desc: ClassVar[str]

class StatsdHost(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    default: ClassVar[None]
    validator: ClassVar[_AddressValidatorType]
    desc: ClassVar[str]

class DogstatsdTags(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    default: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    desc: ClassVar[str]

class StatsdPrefix(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    default: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    desc: ClassVar[str]

class BacklogMetric(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    default: ClassVar[bool]
    action: ClassVar[str]
    desc: ClassVar[str]

class Procname(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class DefaultProcName(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class PythonPath(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class Paste(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class OnStarting(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_OnStartingHookType]
    desc: ClassVar[str]

    def on_starting(server: Arbiter) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class OnReload(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_OnReloadHookType]
    desc: ClassVar[str]

    def on_reload(server: Arbiter) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class WhenReady(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_WhenReadyHookType]
    desc: ClassVar[str]

    def when_ready(server: Arbiter) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class Prefork(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_PreForkHookType]
    desc: ClassVar[str]

    def pre_fork(server: Arbiter, worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class Postfork(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_PostForkHookType]
    desc: ClassVar[str]

    def post_fork(server: Arbiter, worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class PostWorkerInit(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_PostWorkerInitHookType]
    desc: ClassVar[str]

    def post_worker_init(worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class WorkerInt(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_WorkerIntHookType]
    desc: ClassVar[str]

    def worker_int(worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class WorkerAbort(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_WorkerAbortHookType]
    desc: ClassVar[str]

    def worker_abort(worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class PreExec(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_PreExecHookType]
    desc: ClassVar[str]

    def pre_exec(server: Arbiter) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class PreRequest(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_PreRequestHookType]
    desc: ClassVar[str]

    def pre_request(worker: Worker, req: Request) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class PostRequest(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_PostRequestHookType]
    desc: ClassVar[str]

    def post_request(worker: Worker, req: Request, environ: _EnvironType, resp: Response) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class ChildExit(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_ChildExitHookType]
    desc: ClassVar[str]

    def child_exit(server: Arbiter, worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class WorkerExit(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_WorkerExitHookType]
    desc: ClassVar[str]

    def worker_exit(server: Arbiter, worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class NumWorkersChanged(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_NumWorkersChangedHookType]
    desc: ClassVar[str]

    def nworkers_changed(server: Arbiter, new_value: int, old_value: int | None) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class OnExit(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    default: ClassVar[_OnExitHookType]
    desc: ClassVar[str]

    def on_exit(server: Arbiter) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class NewSSLContext(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_SSLContextHookType]
    desc: ClassVar[str]

    def ssl_context(config: Config, default_ssl_context_factory: Callable[[], SSLContext]) -> SSLContext: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

def validate_proxy_protocol(val: str | bool | None) -> str: ...

class ProxyProtocol(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ProxyProtocolValidatorType]
    default: ClassVar[str]
    nargs: ClassVar[str]
    const: ClassVar[str]
    desc: ClassVar[str]

class ProxyAllowFrom(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class Protocol(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class UWSGIAllowFrom(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class KeyFile(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class CertFile(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class SSLVersion(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[Callable[[_SSLMethod], _SSLMethod]]
    default: ClassVar[_SSLMethod]
    desc: ClassVar[str]

class CertReqs(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_IntValidatorType]
    default: ClassVar[int]
    desc: ClassVar[str]

class CACerts(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class SuppressRaggedEOFs(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    action: ClassVar[str]
    default: ClassVar[bool]
    validator: ClassVar[_BoolValidatorType]
    desc: ClassVar[str]

class DoHandshakeOnConnect(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class Ciphers(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

VALID_HTTP_PROTOCOLS: Final[frozenset[str]]
ALPN_PROTOCOL_MAP: Final[dict[str, str]]

def validate_http_protocols(val: str | None) -> list[str]: ...

class HTTPProtocols(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_HTTPProtocolsValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class HTTP2MaxConcurrentStreams(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class HTTP2InitialWindowSize(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class HTTP2MaxFrameSize(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_HTTP2FrameSizeValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class HTTP2MaxHeaderListSize(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class PasteGlobalConf(Setting):
    name: ClassVar[str]
    action: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[list[str]]
    desc: ClassVar[str]

class PermitObsoleteFolding(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class StripHeaderSpaces(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class PermitUnconventionalHTTPMethod(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class PermitUnconventionalHTTPVersion(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class CasefoldHTTPMethod(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class ForwarderHeaders(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class HeaderMap(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

def validate_asgi_loop(val: str | None) -> str: ...
def validate_asgi_lifespan(val: str | None) -> str: ...
def validate_http_parser(val: str | None) -> str: ...

class ASGILoop(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ASGILoopValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class ASGILifespan(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ASGILifespanValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class ASGIDisconnectGracePeriod(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class HttpParser(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_HttpParserValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class RootPath(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class DirtyApps(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    action: ClassVar[str]
    meta: ClassVar[str]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[list[str]]
    desc: ClassVar[str]

class DirtyWorkers(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class DirtyTimeout(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class DirtyThreads(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class DirtyGracefulTimeout(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class OnDirtyStarting(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_OnDirtyStartingHookType]
    desc: ClassVar[str]

    def on_dirty_starting(arbiter: Arbiter) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class DirtyPostFork(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_DirtyPostForkHookType]
    desc: ClassVar[str]

    def dirty_post_fork(arbiter: Arbiter, worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class DirtyWorkerInit(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_DirtyWorkerInitHookType]
    desc: ClassVar[str]

    def dirty_worker_init(worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class DirtyWorkerExit(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_DirtyWorkerExitHookType]
    desc: ClassVar[str]

    def dirty_worker_exit(arbiter: Arbiter, worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class ControlSocket(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    default_doc: ClassVar[str]
    desc: ClassVar[str]

class ControlSocketMode(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[Callable[[Any, str], int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class ControlSocketDisable(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]



from _typeshed import Incomplete
from asyncio.base_events import Server
from asyncio.events import AbstractEventLoop
from typing import NoReturn

from gunicorn.asgi.lifespan import LifespanManager
from gunicorn.config import Config
from gunicorn.glogging import Logger as GLogger
from gunicorn.workers import base

from .._types import _ASGIAppType

class ASGIWorker(base.Worker):
    worker_connections: int
    loop: AbstractEventLoop | None
    servers: list[Server]
    nr_conns: int
    lifespan: LifespanManager | None
    state: dict[Incomplete, Incomplete]
    asgi: _ASGIAppType

    @classmethod
    def check_config(cls, cfg: Config, log: GLogger) -> None: ...
    def init_process(self) -> None: ...
    def load_wsgi(self) -> None: ...
    def init_signals(self) -> None: ...
    def handle_quit_signal(self) -> None: ...
    def handle_exit_signal(self) -> None: ...
    def handle_usr1_signal(self) -> None: ...
    def handle_winch_signal(self) -> None: ...
    def handle_abort_signal(self) -> NoReturn: ...
    def run(self) -> None: ...



from typing import Any, TypeVar

from google.protobuf.descriptor_pool import DescriptorPool
from google.protobuf.message import Message

_MessageT = TypeVar("_MessageT", bound=Message)

def serialize(
    message: Message,
    always_print_fields_with_no_presence: bool = False,
    preserving_proto_field_name: bool = False,
    use_integers_for_enums: bool = False,
    descriptor_pool: DescriptorPool | None = None,
) -> dict[str, Any]: ...
def parse(
    message_class: type[_MessageT],
    js_dict: dict[str, Any],
    ignore_unknown_fields: bool = False,
    descriptor_pool: DescriptorPool | None = None,
    max_recursion_depth: int = 100,
) -> _MessageT: ...



from _typeshed import Incomplete

from kafka.protocol.api import Request, Response

class SaslHandshakeResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class SaslHandshakeResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class SaslHandshakeRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = SaslHandshakeResponse_v0
    SCHEMA: Incomplete

class SaslHandshakeRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = SaslHandshakeResponse_v1
    SCHEMA: Incomplete

SaslHandshakeRequest: Incomplete
SaslHandshakeResponse: Incomplete



from _hashlib import HASH
from _typeshed import ReadableBuffer
from collections.abc import Callable
from typing import ClassVar, Final

from paramiko.message import Message
from paramiko.transport import Transport

c_MSG_KEXDH_INIT: Final[bytes]
c_MSG_KEXDH_REPLY: Final[bytes]
b7fffffffffffffff: Final[bytes]
b0000000000000000: Final[bytes]

class KexGroup14SHA256:
    P: ClassVar[int]
    G: ClassVar[int]

    name: ClassVar[str]
    hash_algo: ClassVar[Callable[[ReadableBuffer], HASH]]

    transport: Transport
    x: int
    e: int
    f: int

    def __init__(self, transport: Transport) -> None: ...
    def start_kex(self) -> None: ...
    def parse_next(self, ptype: int, m: Message) -> None: ...



from _typeshed import Incomplete

from kafka.protocol.api import Request, Response

class OffsetForLeaderEpochResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetForLeaderEpochResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetForLeaderEpochResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetForLeaderEpochResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetForLeaderEpochResponse_v4(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetForLeaderEpochRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetForLeaderEpochResponse_v0
    SCHEMA: Incomplete

class OffsetForLeaderEpochRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetForLeaderEpochResponse_v1
    SCHEMA: Incomplete

class OffsetForLeaderEpochRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetForLeaderEpochResponse_v2
    SCHEMA: Incomplete

class OffsetForLeaderEpochRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetForLeaderEpochResponse_v3
    SCHEMA: Incomplete

class OffsetForLeaderEpochRequest_v4(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetForLeaderEpochResponse_v4
    SCHEMA: Incomplete

OffsetForLeaderEpochRequest: Incomplete
OffsetForLeaderEpochResponse: Incomplete



from kafka.metrics.measurable_stat import AbstractMeasurableStat

class Total(AbstractMeasurableStat):
    def __init__(self, value: float = 0.0) -> None: ...
    def record(self, config, value, now) -> None: ...
    def measure(self, config, now): ...



import sys

# sys.platform.startswith(("freebsd", "midnightbsd", "openbsd", "netbsd")):
if sys.platform != "linux" and sys.platform != "win32" and sys.platform != "darwin":
    from _typeshed import Incomplete
    from collections import defaultdict
    from collections.abc import Callable
    from contextlib import AbstractContextManager
    from typing import Final, NamedTuple, ParamSpec, TypeVar, overload

    from psutil._common import (
        FREEBSD as FREEBSD,
        NETBSD as NETBSD,
        OPENBSD as OPENBSD,
        AccessDenied as AccessDenied,
        NoSuchProcess as NoSuchProcess,
        ZombieProcess as ZombieProcess,
        conn_tmap as conn_tmap,
        conn_to_ntuple as conn_to_ntuple,
        memoize as memoize,
        usage_percent as usage_percent,
    )

    from . import _ntuples as ntp, _psposix, _psutil_bsd

    _P = ParamSpec("_P")
    _R = TypeVar("_R")

    __extra__all__: Final[list[str]]
    PROC_STATUSES: Final[dict[int, str]]
    TCP_STATUSES: Final[dict[int, str]]
    PAGESIZE: Final[int]
    AF_LINK: Final = _psutil_bsd.AF_LINK
    HAS_PROC_NUM_THREADS: Final[bool]
    kinfo_proc_map: Final[dict[str, int]]

    class svmem(NamedTuple):
        total: int
        available: int
        percent: float
        used: int
        free: int
        active: int
        inactive: int
        buffers: int
        cached: int
        shared: int
        wired: int

    class scputimes(NamedTuple):
        user: float
        nice: float
        system: float
        idle: float
        irq: float

    class pmem(NamedTuple):
        rss: int
        vms: int
        text: int
        data: int
        stack: int

    pfullmem = pmem

    class pcputimes(NamedTuple):
        user: float
        system: float
        children_user: float
        children_system: float

    class pmmap_grouped(NamedTuple):
        path: Incomplete
        rss: Incomplete
        private: Incomplete
        ref_count: Incomplete
        shadow_count: Incomplete

    class pmmap_ext(NamedTuple):
        addr: Incomplete
        perms: Incomplete
        path: Incomplete
        rss: Incomplete
        private: Incomplete
        ref_count: Incomplete
        shadow_count: Incomplete

    class sdiskio(NamedTuple):
        read_count: Incomplete
        write_count: Incomplete
        read_bytes: Incomplete
        write_bytes: Incomplete
        read_time: Incomplete
        write_time: Incomplete
        busy_time: Incomplete

    def virtual_memory() -> svmem: ...
    def swap_memory() -> ntp.sswap: ...
    heap_info = _psutil_bsd.heap_info  # only FreeBSD and NetBSD
    heap_trim = _psutil_bsd.heap_trim  # only FreeBSD and NetBSD
    def cpu_times() -> scputimes: ...
    def per_cpu_times() -> list[scputimes]: ...
    def cpu_count_logical() -> int | None: ...
    def cpu_count_cores() -> int | None: ...
    def cpu_stats() -> ntp.scpustats: ...
    def disk_partitions(all: bool = False) -> list[ntp.sdiskpart]: ...

    disk_usage = _psposix.disk_usage
    disk_io_counters = _psutil_bsd.disk_io_counters
    net_io_counters = _psutil_bsd.net_io_counters
    net_if_addrs = _psutil_bsd.net_if_addrs

    def net_if_stats() -> dict[str, ntp.snicstats]: ...
    def net_connections(kind: str) -> list[ntp.sconn]: ...
    def sensors_battery() -> ntp.sbattery | None: ...  # only FreeBSD
    def sensors_temperatures() -> defaultdict[str, list[ntp.shwtemp]]: ...  # only FreeBSD
    def cpu_freq() -> list[ntp.scpufreq]: ...  # only FreeBSD and OpenBSD
    def boot_time() -> float: ...
    def users() -> list[ntp.suser]: ...

    INIT_BOOT_TIME: Final[float]  # only NetBSD

    def adjust_proc_create_time(ctime: float) -> float: ...  # only NetBSD
    def pids() -> list[int]: ...
    def pid_exists(pid: int) -> bool: ...
    def wrap_exceptions(fun: Callable[_P, _R]) -> Callable[_P, _R]: ...
    def wrap_exceptions_procfs(inst: Process) -> AbstractContextManager[None]: ...

    class Process:
        __slots__ = ["_cache", "_name", "_ppid", "pid"]
        pid: int
        def __init__(self, pid: int) -> None: ...
        def oneshot(
            self,
        ) -> tuple[
            int,
            int,
            int,
            int,
            int,
            int,
            int,
            int,
            int,
            float,
            int,
            int,
            int,
            int,
            float,
            float,
            float,
            float,
            int,
            int,
            int,
            int,
            int,
            int,
            str,
        ]: ...
        def oneshot_enter(self) -> None: ...
        def oneshot_exit(self) -> None: ...
        def name(self) -> str: ...
        def exe(self) -> str: ...
        def cmdline(self) -> list[str]: ...
        def environ(self) -> dict[str, str]: ...
        def terminal(self) -> str | None: ...
        def ppid(self) -> int: ...
        def uids(self) -> ntp.puids: ...
        def gids(self) -> ntp.pgids: ...
        def cpu_times(self) -> ntp.pcputimes: ...
        def cpu_num(self) -> int: ...  # only FreeBSD
        def memory_info(self) -> pmem: ...
        memory_full_info = memory_info
        def create_time(self, monotonic: bool = False) -> float: ...
        def num_threads(self) -> int: ...
        def num_ctx_switches(self) -> ntp.pctxsw: ...
        def threads(self) -> list[ntp.pthread]: ...
        def net_connections(self, kind: str = "inet") -> list[ntp.pconn]: ...
        def wait(self, timeout: float | None = None) -> int | None: ...
        def nice_get(self) -> int: ...
        def nice_set(self, value: int) -> None: ...
        def status(self) -> str: ...
        def io_counters(self) -> ntp.pio: ...
        def cwd(self) -> str: ...

        class nt_mmap_grouped(NamedTuple):
            path: Incomplete
            rss: Incomplete
            private: Incomplete
            ref_count: Incomplete
            shadow_count: Incomplete

        class nt_mmap_ext(NamedTuple):
            addr: Incomplete
            perms: Incomplete
            path: Incomplete
            rss: Incomplete
            private: Incomplete
            ref_count: Incomplete
            shadow_count: Incomplete

        def open_files(self) -> list[ntp.popenfile]: ...
        def num_fds(self) -> int: ...
        def cpu_affinity_get(self) -> list[int]: ...  # only FreeBSD
        def cpu_affinity_set(self, cpus: list[int]) -> None: ...  # only FreeBSD
        def memory_maps(self) -> list[tuple[str, str, str, int, int, int, int]]: ...  # only FreeBSD

        @overload
        def rlimit(self, resource: int, limits: tuple[int, int]) -> None: ...  # only FreeBSD
        @overload
        def rlimit(self, resource: int, limits: None = None) -> tuple[int, int]: ...  # only FreeBSD



from logging import Logger
from typing import Final

from .decorator import cross_origin as cross_origin
from .extension import CORS as CORS

__version__: Final[str]

rootlogger: Logger

__all__ = ["CORS", "__version__", "cross_origin"]



import asyncio
from collections.abc import Iterable
from typing import Final, Literal, TypedDict, type_check_only
from typing_extensions import NotRequired

from gunicorn.asgi.parser import CallbackRequest
from gunicorn.config import Config
from gunicorn.glogging import Logger as GLogger
from gunicorn.workers.gasgi import ASGIWorker

from .._types import _ASGIAppType

HIGH_WATER_LIMIT: Final = 65536

class FlowControl:
    __slots__ = ("_transport", "read_paused", "write_paused", "_is_writable_event")
    read_paused: bool
    write_paused: bool

    def __init__(self, transport: asyncio.BaseTransport) -> None: ...
    async def drain(self) -> None: ...
    def pause_reading(self) -> None: ...
    def resume_reading(self) -> None: ...
    def pause_writing(self) -> None: ...
    def resume_writing(self) -> None: ...

class ASGIResponseInfo:
    status: str | int
    sent: int
    headers: list[tuple[str, str]]

    def __init__(self, status: str | int, headers: Iterable[tuple[str | bytes, str | bytes]], sent: int) -> None: ...

@type_check_only
class _BodyReceieverReceiveReturnType(TypedDict):
    type: Literal["http.disconnect", "http.request"]
    body: NotRequired[bytes]
    more_body: NotRequired[bool]

class BodyReceiver:
    __slots__ = ("_chunks", "_complete", "_body_finished", "_closed", "_body_wait_expired", "_waiter", "request", "protocol")
    request: CallbackRequest
    protocol: ASGIProtocol

    def __init__(self, request: CallbackRequest, protocol: ASGIProtocol) -> None: ...
    def feed(self, chunk: bytes) -> None: ...
    def set_complete(self) -> None: ...
    def signal_disconnect(self) -> None: ...
    async def receive(self) -> _BodyReceieverReceiveReturnType: ...

class ASGIProtocol(asyncio.Protocol):
    worker: ASGIWorker
    cfg: Config
    log: GLogger
    app: _ASGIAppType
    transport: asyncio.BaseTransport | None
    reader: asyncio.StreamReader | None
    writer: asyncio.BaseTransport | None
    req_count: int

    def __init__(self, worker: ASGIWorker) -> None: ...



import logging
import sys
import unittest.result
from _typeshed import SupportsDunderGE, SupportsDunderGT, SupportsDunderLE, SupportsDunderLT, SupportsRSub, SupportsSub
from builtins import _ClassInfo
from collections.abc import Callable, Container, Iterable, Mapping, Sequence, Set as AbstractSet
from contextlib import AbstractContextManager
from re import Pattern
from types import GenericAlias, TracebackType
from typing import (
    Any,
    AnyStr,
    Final,
    Generic,
    NoReturn,
    ParamSpec,
    Protocol,
    SupportsAbs,
    SupportsRound,
    TypeVar,
    overload,
    type_check_only,
)
from typing_extensions import Never, Self
from unittest._log import _AssertLogsContext, _LoggingWatcher
from warnings import WarningMessage

_T = TypeVar("_T")
_S = TypeVar("_S", bound=SupportsSub[Any, Any])
_E = TypeVar("_E", bound=BaseException)
_FT = TypeVar("_FT", bound=Callable[..., Any])
_SB = TypeVar("_SB", str, bytes, bytearray)
_P = ParamSpec("_P")

DIFF_OMITTED: Final[str]

class _BaseTestCaseContext:
    test_case: TestCase
    def __init__(self, test_case: TestCase) -> None: ...

class _AssertRaisesBaseContext(_BaseTestCaseContext):
    expected: type[BaseException] | tuple[type[BaseException], ...]
    expected_regex: Pattern[str] | None
    obj_name: str | None
    msg: str | None

    def __init__(
        self,
        expected: type[BaseException] | tuple[type[BaseException], ...],
        test_case: TestCase,
        expected_regex: str | Pattern[str] | None = None,
    ) -> None: ...

    # This returns Self if args is the empty list, and None otherwise.
    # but it's not possible to construct an overload which expresses that
    def handle(self, name: str, args: list[Any], kwargs: dict[str, Any]) -> Any: ...

def addModuleCleanup(function: Callable[_P, object], /, *args: _P.args, **kwargs: _P.kwargs) -> None: ...
def doModuleCleanups() -> None: ...

if sys.version_info >= (3, 11):
    def enterModuleContext(cm: AbstractContextManager[_T]) -> _T: ...

def expectedFailure(test_item: _FT) -> _FT: ...
def skip(reason: str) -> Callable[[_FT], _FT]: ...
def skipIf(condition: object, reason: str) -> Callable[[_FT], _FT]: ...
def skipUnless(condition: object, reason: str) -> Callable[[_FT], _FT]: ...

class SkipTest(Exception):
    def __init__(self, reason: str, /) -> None: ...

@type_check_only
class _SupportsAbsAndDunderGE(SupportsDunderGE[Any], SupportsAbs[Any], Protocol): ...

class TestCase:
    failureException: type[BaseException]
    longMessage: bool
    maxDiff: int | None
    # undocumented
    _testMethodName: str
    # undocumented
    _testMethodDoc: str
    def __init__(self, methodName: str = "runTest") -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def setUp(self) -> None: ...
    def tearDown(self) -> None: ...
    @classmethod
    def setUpClass(cls) -> None: ...
    @classmethod
    def tearDownClass(cls) -> None: ...
    def run(self, result: unittest.result.TestResult | None = None) -> unittest.result.TestResult | None: ...
    def __call__(self, result: unittest.result.TestResult | None = ...) -> unittest.result.TestResult | None: ...
    def skipTest(self, reason: Any) -> NoReturn: ...
    def subTest(self, msg: Any = ..., **params: Any) -> AbstractContextManager[None]: ...
    def debug(self) -> None: ...
    if sys.version_info < (3, 11):
        def _addSkip(self, result: unittest.result.TestResult, test_case: TestCase, reason: str) -> None: ...

    def assertEqual(self, first: Any, second: Any, msg: Any = None) -> None: ...
    def assertNotEqual(self, first: Any, second: Any, msg: Any = None) -> None: ...
    def assertTrue(self, expr: Any, msg: Any = None) -> None: ...
    def assertFalse(self, expr: Any, msg: Any = None) -> None: ...
    def assertIs(self, expr1: object, expr2: object, msg: Any = None) -> None: ...
    def assertIsNot(self, expr1: object, expr2: object, msg: Any = None) -> None: ...
    def assertIsNone(self, obj: object, msg: Any = None) -> None: ...
    def assertIsNotNone(self, obj: object, msg: Any = None) -> None: ...
    def assertIn(self, member: Any, container: Iterable[Any] | Container[Any], msg: Any = None) -> None: ...
    def assertNotIn(self, member: Any, container: Iterable[Any] | Container[Any], msg: Any = None) -> None: ...
    def assertIsInstance(self, obj: object, cls: _ClassInfo, msg: Any = None) -> None: ...
    def assertNotIsInstance(self, obj: object, cls: _ClassInfo, msg: Any = None) -> None: ...

    @overload
    def assertGreater(self, a: SupportsDunderGT[_T], b: _T, msg: Any = None) -> None: ...
    @overload
    def assertGreater(self, a: _T, b: SupportsDunderLT[_T], msg: Any = None) -> None: ...

    @overload
    def assertGreaterEqual(self, a: SupportsDunderGE[_T], b: _T, msg: Any = None) -> None: ...
    @overload
    def assertGreaterEqual(self, a: _T, b: SupportsDunderLE[_T], msg: Any = None) -> None: ...

    @overload
    def assertLess(self, a: SupportsDunderLT[_T], b: _T, msg: Any = None) -> None: ...
    @overload
    def assertLess(self, a: _T, b: SupportsDunderGT[_T], msg: Any = None) -> None: ...

    @overload
    def assertLessEqual(self, a: SupportsDunderLE[_T], b: _T, msg: Any = None) -> None: ...
    @overload
    def assertLessEqual(self, a: _T, b: SupportsDunderGE[_T], msg: Any = None) -> None: ...

    # `assertRaises`, `assertRaisesRegex`, and `assertRaisesRegexp`
    # are not using `ParamSpec` intentionally,
    # because they might be used with explicitly wrong arg types to raise some error in tests.
    @overload
    def assertRaises(
        self,
        expected_exception: type[BaseException] | tuple[type[BaseException], ...],
        callable: Callable[..., object],
        *args: Any,
        **kwargs: Any,
    ) -> None: ...
    @overload
    def assertRaises(
        self, expected_exception: type[_E] | tuple[type[_E], ...], *, msg: Any = ...
    ) -> _AssertRaisesContext[_E]: ...

    @overload
    def assertRaisesRegex(
        self,
        expected_exception: type[BaseException] | tuple[type[BaseException], ...],
        expected_regex: str | Pattern[str],
        callable: Callable[..., object],
        *args: Any,
        **kwargs: Any,
    ) -> None: ...
    @overload
    def assertRaisesRegex(
        self, expected_exception: type[_E] | tuple[type[_E], ...], expected_regex: str | Pattern[str], *, msg: Any = ...
    ) -> _AssertRaisesContext[_E]: ...

    @overload
    def assertWarns(
        self,
        expected_warning: type[Warning] | tuple[type[Warning], ...],
        callable: Callable[_P, object],
        *args: _P.args,
        **kwargs: _P.kwargs,
    ) -> None: ...
    @overload
    def assertWarns(
        self, expected_warning: type[Warning] | tuple[type[Warning], ...], *, msg: Any = ...
    ) -> _AssertWarnsContext: ...

    @overload
    def assertWarnsRegex(
        self,
        expected_warning: type[Warning] | tuple[type[Warning], ...],
        expected_regex: str | Pattern[str],
        callable: Callable[_P, object],
        *args: _P.args,
        **kwargs: _P.kwargs,
    ) -> None: ...
    @overload
    def assertWarnsRegex(
        self, expected_warning: type[Warning] | tuple[type[Warning], ...], expected_regex: str | Pattern[str], *, msg: Any = ...
    ) -> _AssertWarnsContext: ...

    if sys.version_info >= (3, 15):
        def assertLogs(
            self,
            logger: str | logging.Logger | None = None,
            level: int | str | None = None,
            formatter: logging.Formatter | None = None,
        ) -> _AssertLogsContext[_LoggingWatcher]: ...
    else:
        def assertLogs(
            self, logger: str | logging.Logger | None = None, level: int | str | None = None
        ) -> _AssertLogsContext[_LoggingWatcher]: ...

    def assertNoLogs(
        self, logger: str | logging.Logger | None = None, level: int | str | None = None
    ) -> _AssertLogsContext[None]: ...

    @overload
    def assertAlmostEqual(self, first: _S, second: _S, places: None, msg: Any, delta: _SupportsAbsAndDunderGE) -> None: ...
    @overload
    def assertAlmostEqual(
        self, first: _S, second: _S, places: None = None, msg: Any = None, *, delta: _SupportsAbsAndDunderGE
    ) -> None: ...
    @overload
    def assertAlmostEqual(
        self,
        first: SupportsSub[_T, SupportsAbs[SupportsRound[object]]],
        second: _T,
        places: int | None = None,
        msg: Any = None,
        delta: None = None,
    ) -> None: ...
    @overload
    def assertAlmostEqual(
        self,
        first: _T,
        second: SupportsRSub[_T, SupportsAbs[SupportsRound[object]]],
        places: int | None = None,
        msg: Any = None,
        delta: None = None,
    ) -> None: ...

    @overload
    def assertNotAlmostEqual(self, first: _S, second: _S, places: None, msg: Any, delta: _SupportsAbsAndDunderGE) -> None: ...
    @overload
    def assertNotAlmostEqual(
        self, first: _S, second: _S, places: None = None, msg: Any = None, *, delta: _SupportsAbsAndDunderGE
    ) -> None: ...
    @overload
    def assertNotAlmostEqual(
        self,
        first: SupportsSub[_T, SupportsAbs[SupportsRound[object]]],
        second: _T,
        places: int | None = None,
        msg: Any = None,
        delta: None = None,
    ) -> None: ...
    @overload
    def assertNotAlmostEqual(
        self,
        first: _T,
        second: SupportsRSub[_T, SupportsAbs[SupportsRound[object]]],
        places: int | None = None,
        msg: Any = None,
        delta: None = None,
    ) -> None: ...

    def assertRegex(self, text: AnyStr, expected_regex: AnyStr | Pattern[AnyStr], msg: Any = None) -> None: ...
    def assertNotRegex(self, text: AnyStr, unexpected_regex: AnyStr | Pattern[AnyStr], msg: Any = None) -> None: ...
    def assertCountEqual(self, first: Iterable[Any], second: Iterable[Any], msg: Any = None) -> None: ...
    def addTypeEqualityFunc(self, typeobj: type[Any], function: Callable[..., None]) -> None: ...
    def assertMultiLineEqual(self, first: str, second: str, msg: Any = None) -> None: ...
    def assertSequenceEqual(
        self, seq1: Sequence[Any], seq2: Sequence[Any], msg: Any = None, seq_type: type[Sequence[Any]] | None = None
    ) -> None: ...
    def assertListEqual(self, list1: list[Any], list2: list[Any], msg: Any = None) -> None: ...
    def assertTupleEqual(self, tuple1: tuple[Any, ...], tuple2: tuple[Any, ...], msg: Any = None) -> None: ...
    def assertSetEqual(self, set1: AbstractSet[object], set2: AbstractSet[object], msg: Any = None) -> None: ...
    # assertDictEqual accepts only true dict instances. We can't use that here, since that would make
    # assertDictEqual incompatible with TypedDict.
    def assertDictEqual(self, d1: Mapping[Any, object], d2: Mapping[Any, object], msg: Any = None) -> None: ...
    def fail(self, msg: Any = None) -> NoReturn: ...
    def countTestCases(self) -> int: ...
    def defaultTestResult(self) -> unittest.result.TestResult: ...
    def id(self) -> str: ...
    def shortDescription(self) -> str | None: ...
    def addCleanup(self, function: Callable[_P, object], /, *args: _P.args, **kwargs: _P.kwargs) -> None: ...

    if sys.version_info >= (3, 11):
        def enterContext(self, cm: AbstractContextManager[_T]) -> _T: ...

    def doCleanups(self) -> None: ...
    @classmethod
    def addClassCleanup(cls, function: Callable[_P, object], /, *args: _P.args, **kwargs: _P.kwargs) -> None: ...
    @classmethod
    def doClassCleanups(cls) -> None: ...

    if sys.version_info >= (3, 11):
        @classmethod
        def enterClassContext(cls, cm: AbstractContextManager[_T]) -> _T: ...

    def _formatMessage(self, msg: str | None, standardMsg: str) -> str: ...  # undocumented
    def _getAssertEqualityFunc(self, first: Any, second: Any) -> Callable[..., None]: ...  # undocumented
    if sys.version_info < (3, 12):
        failUnlessEqual = assertEqual
        assertEquals = assertEqual
        failIfEqual = assertNotEqual
        assertNotEquals = assertNotEqual
        failUnless = assertTrue
        assert_ = assertTrue
        failIf = assertFalse
        failUnlessRaises = assertRaises
        failUnlessAlmostEqual = assertAlmostEqual
        assertAlmostEquals = assertAlmostEqual
        failIfAlmostEqual = assertNotAlmostEqual
        assertNotAlmostEquals = assertNotAlmostEqual
        assertRegexpMatches = assertRegex
        assertNotRegexpMatches = assertNotRegex
        assertRaisesRegexp = assertRaisesRegex
        def assertDictContainsSubset(
            self, subset: Mapping[Any, Any], dictionary: Mapping[Any, Any], msg: object = None
        ) -> None: ...

    # Runtime has *args, **kwargs, but will error if any are supplied
    def __init_subclass__(cls, *args: Never, **kwargs: Never) -> None: ...

    if sys.version_info >= (3, 14):
        def assertIsSubclass(self, cls: type, superclass: type | tuple[type, ...], msg: Any = None) -> None: ...
        def assertNotIsSubclass(self, cls: type, superclass: type | tuple[type, ...], msg: Any = None) -> None: ...
        def assertHasAttr(self, obj: object, name: str, msg: Any = None) -> None: ...
        def assertNotHasAttr(self, obj: object, name: str, msg: Any = None) -> None: ...
        def assertStartsWith(self, s: _SB, prefix: _SB | tuple[_SB, ...], msg: Any = None) -> None: ...
        def assertNotStartsWith(self, s: _SB, prefix: _SB | tuple[_SB, ...], msg: Any = None) -> None: ...
        def assertEndsWith(self, s: _SB, suffix: _SB | tuple[_SB, ...], msg: Any = None) -> None: ...
        def assertNotEndsWith(self, s: _SB, suffix: _SB | tuple[_SB, ...], msg: Any = None) -> None: ...

class FunctionTestCase(TestCase):
    def __init__(
        self,
        testFunc: Callable[[], object],
        setUp: Callable[[], object] | None = None,
        tearDown: Callable[[], object] | None = None,
        description: str | None = None,
    ) -> None: ...
    def runTest(self) -> None: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...

class _AssertRaisesContext(_AssertRaisesBaseContext, Generic[_E]):
    exception: _E
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, tb: TracebackType | None
    ) -> bool: ...
    def __class_getitem__(cls, item: Any, /) -> GenericAlias: ...

class _AssertWarnsContext(_AssertRaisesBaseContext):
    warning: WarningMessage
    filename: str
    lineno: int
    warnings: list[WarningMessage]
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, tb: TracebackType | None
    ) -> None: ...



class Histogram:
    def __init__(self, bin_scheme) -> None: ...
    def record(self, value) -> None: ...
    def value(self, quantile): ...
    @property
    def counts(self): ...
    def clear(self) -> None: ...

    class ConstantBinScheme:
        def __init__(self, bins, min_val, max_val) -> None: ...
        @property
        def bins(self): ...
        def from_bin(self, b): ...
        def to_bin(self, x): ...

    class LinearBinScheme:
        def __init__(self, num_bins, max_val) -> None: ...
        @property
        def bins(self): ...
        def from_bin(self, b): ...
        def to_bin(self, x): ...



import pythoncom

class COMException(pythoncom.com_error):
    scode: int
    description: str
    source: str | None
    helpfile: str | None
    helpcontext: int | None
    def __init__(
        self,
        description: str | None = None,
        scode: int | None = None,
        source: str | None = None,
        helpfile: str | None = None,
        helpContext: int | None = None,
        desc: str | None = None,
        hresult: int | None = None,
    ) -> None: ...

def IsCOMException(t: type[BaseException] | None = None) -> bool: ...
def IsCOMServerException(t: type[BaseException] | None = None) -> bool: ...



from collections.abc import Callable

from .compilers.C import cygwin
from .compilers.C.cygwin import (
    CONFIG_H_NOTOK as CONFIG_H_NOTOK,
    CONFIG_H_OK as CONFIG_H_OK,
    CONFIG_H_UNCERTAIN as CONFIG_H_UNCERTAIN,
    check_config_h as check_config_h,
    get_msvcr as get_msvcr,
    is_cygwincc as is_cygwincc,
)
from .version import LooseVersion

__all__ = [
    "CONFIG_H_NOTOK",
    "CONFIG_H_OK",
    "CONFIG_H_UNCERTAIN",
    "CygwinCCompiler",
    "Mingw32CCompiler",
    "check_config_h",
    "get_msvcr",
    "is_cygwincc",
]

CygwinCCompiler = cygwin.Compiler
Mingw32CCompiler = cygwin.MinGW32Compiler

get_versions: Callable[[], tuple[LooseVersion | None, ...]] | None



from _typeshed import Incomplete

from authlib.oauth1 import ResourceProtector as _ResourceProtector

class ResourceProtector(_ResourceProtector):
    app: Incomplete
    query_client: Incomplete
    query_token: Incomplete
    def __init__(self, app=None, query_client=None, query_token=None, exists_nonce=None) -> None: ...
    SUPPORTED_SIGNATURE_METHODS: Incomplete
    def init_app(self, app, query_client=None, query_token=None, exists_nonce=None): ...
    def get_client_by_id(self, client_id): ...
    def get_token_credential(self, request): ...
    def exists_nonce(self, nonce, request): ...
    def acquire_credential(self): ...
    def __call__(self, scope=None): ...

current_credential: Incomplete



from _typeshed import Incomplete

class SortedSet:
    def __init__(self, iterable=None, key=None) -> None: ...
    def first(self): ...
    def last(self): ...
    def pop_last(self): ...
    def add(self, value): ...
    def remove(self, value): ...
    def __contains__(self, value) -> bool: ...
    def __iter__(self): ...
    __nonzero__: Incomplete
    __bool__: Incomplete
